unit PrintPrev;

{
Created by: James "Woody" Woodard
  2/24/99

  The origins of this component came from the Inprise site and contained most
  of the functionality. I don't remember who the author was but thank him for
  the insight and the main procedures for printing and calculating printer
  metrics. The rest of the implementation is my own and some of the original
  procedures were changed to reflect the addition of the preview mode.

  This component is used as a reporting tool when standard report
  generators don't allow you the flexibility needed for more complex printing.
  You can use this component to provide a more robust method of generating
  reports.  Though the amount of work involved in creating a report requires
  more coding than using a report generator, the amount of control you have far
  outweighs the hassles, in my opinion.

  I built this component because I have written several programs that needed to
  be flexible in not only what was printed but where depending on certain
  criteria selectable by the user. Also, most report generators don't allow
  you to expand or contract sections unless you use grids, etc.

  The way this component works is this:
    Initially, when you issue all of the print commands from start to finish,
    they are stored in an object called Pages. This stores all print commands
    in lists, one list per page. This list of pages and commands is then
    replayed either to the printer or to the preview screen. By capturing the
    commands this way, it is possible for the user to select a printing range
    of pages instead of the complete job if desired.

  Modifications by Doug Baird  10/18/2000

  The interface defined in the example on the Inprise site has been replaced
  by an interface that is more word-processor-like.  The functionality has
  been enhanced.

  The public variable Running and the published variable ShowCancel have been
  added to TPrintPreview.  When ShowCancel is True, a cancel button is avaialble
  with the progress bar.  When this button is clicked, Running becomes False.
  By testing Running the print routine can sense this and take a desired action.

  Margins are established for the paper defined to the printer either at
  design time, through the SetPapaerSize procedure or the Printer Setup
  Dialog.  A procedure GetMargins is added retrieve changes made during
  the execution of SetMargins to meet offset requiremets.  Procedures
  TestMargins and FixMargins and the event OnMarginError are added to
  faciliate error handling.

  Added AfterPrinterChange event to adjustments to page layout that might
  be required when the printer is changed using the PrintDialog from the
  preview form (ShowPrintDialog is True).

  GetColumnsPerLine and GetLinesPerPage have been changed to give the areas
  inside the margins.  As originally defined, the offset values are zero and
  the functions do not produce useful results.

  Modified Write commands, reducung the number and addinga capbility to specify
  style of text being written.

  Modified SetHeaderInformation, SetFooterInformation, SetHeaderDimensions and
  SetFooterDimensions so that Y positions are relative to top or bottom
  margins.  If the text in the header or footer information methods conatins
  '%u' then it is assumed that it is an editing string for page numbering.

}
interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, ExtCtrls, Math, Printers, PrintPv1, PrintPv2;

const
  HeaderLines = 10;     { Number of allowable header lines }
  FooterLines = 10;     { Number of allowable footer lines }
  Columns = 20;         { Number of allowable columns }
  wtNextX = -1;         { Used with WriteText to append text }
  wtLeftX = -2;         { Used with WriteText to use left margin }
  wtNextY = -1;         { Used with all Writes to use next line }
  wtThisY = -2;         { Used with all Writes to append text}
  wtLeft = 0;           { Used with WriteColumn/Line for alignment }
  wtCenter = 1;         { Used with WriteColumn/Line for alignment }
  wtRight = 2;          { Used with WriteColumn/Line for alignment }
  wtIndent = 3;         { Used with WriteLine for alignment }
  wtDecimal = 4;        { Used with _Write for decimal alignment }

type
  THeaderRecord = record
    Text: string[240];                   { Header text }
    YPosition: Single;                   { Inches from the top }
    Alignment: Integer;                  { 0:Left 1:Center 2:Right }
    FontName: string[80];                { Font name }
    FontSize: Integer;                   { Font size }
    FontStyle: TFontStyles;              { Font style }
  end;

  TFooterRecord = record
    Text: string[240];                   { Footer text }
    YPosition: Single;                   { Inches from the top }
    Alignment: Integer;                  { 0:Left 1:Center 2:Right }
    FontName: string[80];                { Font name }
    FontSize: Integer;                   { Font size }
    FontStyle: TFontStyles;              { Font style }
  end;

  THeaderCoordinates = record
    XTop: Single;                        { Top left coordinates }
    YTop: Single;
    XBottom: Single;                     { Bottom right coordinates }
    YBottom: Single;
    Boxed: Boolean;                      { Drawing parameters }
    Shading: Word;
    LineWidth: Single;
  end;

  TFooterCoordinates = record
    XTop: Single;                        { Top left coordinated }
    YTop: Single;
    XBottom: Single;                     { Bottom right coordinates }
    YBottom: Single;
    Boxed: Boolean;                      { Drawing parameters }
    Shading: Byte;
    LineWidth: Single;
  end;

  TColumnInformationRecord = record
    XPosition: Single;                   { Column offset and width }
    Length: Single;
  end;

  TFontInformationRecord = record
    Name: string[30];                    { Font specification }
    Size: Integer;
    Style: TFontStyles;
  end;

  Percentages = (P1_Forty,P2_Fifty,P3_Sixty,P4_Seventy,P5_Eighty,
                 P6_Ninety,P7_Full,P8_Width,P9_Page);

  PaperSizes = (PtrSU, Letter, Legal);

  TPrintPreview = class(TComponent)
  private
    TopMargin: Integer;               { Top margin in pixels }
    BottomMargin: Integer;            { Bottom margin in pixels }
    LeftMargin: Integer;              { Left margin in pixels }
    RightMargin: Integer;             { Right margin in pixels }
    CurrentTab: Integer;              { Left indent in pixels }
    inTopMargin: Single;              { Top margin in inches }
    inBottomMargin: Single;           { Bottom margin in inches }
    inLeftMargin: Single;             { Left margin in inches }
    inRightMargin: Single;            { Right margin in inches }
    inCurrentTab: Single;             { Left indent in inches }
    ResolutionY: Integer;             { Number of pixels per inch
                                        along Y axis }
    ResolutionX: Integer;             { Number of pixels per inch
                                        along X axis }
    PaperWidthPixels: Integer;        { Full width of page in pixels
                                        includes offset }
    PaperHeightPixels: Integer;       { Full height of page in pixels
                                        includes offset }
    PaperHeightInches: Single;        { Height of page in inches }
    PaperWidthInches: Single;         { Width of page in inches }
    OffsetLeft: Integer;              { Unprintable area on left }
    OffsetRight: Integer;             { Unprintable area on right }
    OffsetTop: Integer;               { Unprintable area on top }
    OffsetBottom: Integer;            { Unprintable area on bottom }
    DetailTop: Single;                { Inches from the top where the
                                        detail section starts }
    DetailBottom: Single;             { Inches from the top where the
                                        detail section ends }
    AutoPaging: Boolean;              { Are new pages automatically generated? }
    LastYPosition: Single;            { The Y position where the last
                                        write occurred }
    HelpCntxt: THelpContext;          { Used to set print preview help }
    aDeviceMode: THandle;             { Used with SetPaperSize }
    CurrentFonts: array [1..10] of TFontInformationRecord;
    CurrentFontRecord: Integer;
    TextMetrics: TTextMetric;
    Header: array[1..HeaderLines] of THeaderRecord;
    Footer: array[1..FooterLines] of TFooterRecord;
    ColumnInformation: array[1..Columns] of TColumnInformationRecord;
    HeaderCoordinates: THeaderCoordinates;
    FooterCoordinates: TFooterCoordinates;
    ThePrinter: TImage; {Used for current canvas during print preview mode}
    TheCanvas: TCanvas; {This is set to current drawing canvas}
    DryRun: Boolean;  {used for counting total pages before actually printing report}
    PreviewMode: Boolean; {Used to set internal preview mode to build pages}
    ActualPreview: Boolean; {Used to control users choice of preview mode}
    ScaleByValue: Single; {Used to scale the pixels per inch for scaling}
    UseProgress: Boolean; {Shows progress bar if set to true}
    ProgressBar: TPrintBarForm; {Progress bar form}
    DocumentTitle: string; {Title to show in print manager for current job}
    PageOrientation: TPrinterOrientation; {Keep track of orientation}
    ShowDialog: Boolean; {True to allow print dialog box to show}
    ProgressMax: Word; {Sets the progress bar maximum}
    ProgressInc: Word; {Sets the progress bar increment}
    PrintTransparent: Boolean; {True to print transparent text for backgrounds}
    FPageChange: TNotifyEvent; {Event for user to put any specific printing to
                                be done for every page.}
    FAPageChange: TNotifyEvent; {Event for user to put any specific printing to
                                 be done for every page.}
    FPrinterChange: TNotifyEvent; {Event for user to adjust page formatting
                                   if PrintDialog changes printers where the
                                   paper parameters could impact page layout.}
    FMarginError: TNotifyEvent; { Event for user to adjust margins if set
                                  outside offsets. }
    PreviewPercent: Percentages;
    PaperDimension: PaperSizes;
    procedure OnPageChange;
    procedure AfterPageChange;
    procedure CheckLayout;
    procedure ShowPreview; {call ShowPreview to see the actual report on screen}
    function CalculateLineHeight: Integer;
    procedure CalculateMeasurements;
    procedure DoWrite(Alignment: Word; Y:Single; Text:string; hfFlag: Boolean);
    procedure _Write(Alignment: Integer; X: Single; Y: Single; Text:string;
                     Style:TFontStyles; hfFlag: Boolean);
    procedure _DrawBox(XTop,YTop,XBottom,YBottom:Word;LineWidth:Single;
                        Shading:Word);
    procedure Cleanup;
    function CurrentFontName: string;
    function CurrentFontSize: Word;
    function CurrentFontStyle: TFontStyles;
    function GetShowCancel: Boolean;
    procedure SetShowCancel(SetIt: Boolean);
    function GetRunning: Boolean;
    procedure SetRunning(SetIt: Boolean);
    procedure SetYPosition(YPos: Single);
    procedure PrinterError(Msg: string);
    procedure PrintFromPreview; {used to print from the preview screen}
    procedure PrintPage(Page: TMWPageList;iPage: Integer); {procedure to
          op through the current page commands and output to screen or printer}
    procedure SetPrnPrinter(Current: TImage); {used when printing to TImage}
    function GetScale: Single; {returns the current scale factor}
    procedure SetScale(ScaleFactor: Single); {sets the current scale factor}
    procedure WriteHeader;
    procedure WriteFooter;
    function  GetColumnsPerLine: Integer;
    function  GetLineHeightInches: Single;
    function  GetLineHeightPixels: Word;
    function  GetLinesInDetailArea: Word;
    function  GetLinesLeft: Word;
    function  GetLinesPerPage: Integer;
    function  GetPageNumber:Integer;
  public
    LastXPosition: Single;  { The X position after the last write occurred }
    constructor Create(AOwner: TComponent); override;{need to initialize some default values}
    destructor Destroy; override;
    function GetColumnLeft(Number:Word): Single; {returns start in inches of column}
    function GetColumnRight(Number:Word): Single; {returns end in inches of column}
    procedure PrintGraphic(X,Y,W,H: Single;thePicture: TImage); {Use this procedure to place an
                                                                 image on the printed page.}
    procedure UpdateBar(X: Integer); {used to set progress bar value}
    procedure IncBar(x: Integer); {used to increment the progress bar by X}
    procedure Increment; {used to increment the progress bar by ProgressBarInc value}
    procedure DrawTextBox(XTop,YTop,XBottom,YBottom:Single; LineWidth:Single;
                           Shading:Word; Text: string; Column: Integer;
                           Alignment: Word); {draws a box around written text}
    procedure Start; {starts the print job and initializes things}
    procedure Finish; {must be called to complete the print job}
    procedure Abort; {called to abort the print job}
    function InchesToPixelsX(Inches: Single): Integer;
    function InchesToPixelsY(Inches: Single): Integer;
    function PixelsToInchesX(Pixels: Integer): Single;
    function PixelsToInchesY(Pixels: Integer): Single;
    function LinesToPixels(Line:Integer): Integer;
    procedure GetPaperParms(var Top,Bottom,Left,Right,Height,Width: Single);
    procedure SetMargins(Top,Bottom,Left,Right: Single);
    procedure GetMargins(var Top,Bottom,Left,Right: Single);
    function TestMargins(Top,Bottom,Left,Right: Single): Boolean;
    procedure FixMargins(var Top,Bottom,Left,Right: Single);
    procedure SetFontStyle(Style: TFontStyles);
    procedure SetFontInformation(Name:string; Size:Word; Style: TFontStyles);
    procedure SetTab(Inches: Single);
    procedure WriteLine(Alignment: Word; Y:Single; Text:string);
    procedure WriteColumn(ColumnNumber:Word; Alignment: Word; Y:Single; Text:string);
    procedure WriteDecimal(X:Single; Y:Single; Text:string);
    procedure WriteText(X:Single; Y:Single; Text:string; Style: TFontStyles);
    procedure DrawLine(TopX,TopY,BottomX,BottomY:Single; LineWidth:Single);
    procedure NewPage;
    procedure GetPixelsPerInch(var X:Word; var Y:Word);
    procedure GetPixelsPerPage(var X:Word; var Y:Word);
    function  GetTextWidthPixels(Text:string): Integer;
    function  GetTextWidthInches(Text:string): Single;
    { Reorder parameters in next two methods to conform to Write Line}
    procedure SetHeaderInformation(Line:Integer; Alignment:Word;
                                    YPosition: Single; Text:string;
                                    FontName:string; FontSize: Word;
                                    FontStyle: TFontStyles);
    procedure SetFooterInformation(Line:Integer; Alignment:Word;
                                    YPosition: Single; Text:string;
                                    FontName:string; FontSize: Word;
                                    FontStyle: TFontStyles);
    procedure SaveCurrentFont;
    procedure RestoreCurrentFont;
    procedure SetDetailTopBottom(Top: Single; Bottom: Single);
    procedure DrawBox(XTop,YTop,XBottom,YBottom:Single; LineWidth: Single);
    procedure DrawBoxShaded(XTop,YTop,XBottom,YBottom:Single; LineWidth: Single;
                             Shading:Word);
    procedure SetHeaderDimensions(XTop,YTop,XBottom,YBottom:Single; Boxed: Boolean;
                                   LineWidth: Single; Shading:Word);
    procedure SetFooterDimensions(XTop,YTop,XBottom,YBottom:Single; Boxed: Boolean;
                                   LineWidth: Single; Shading:Word);
    procedure CreateColumn(Number:Word; XPosition:Single; Length:Single);
    procedure NextLine;
    procedure SetTopOfPage;
    procedure NewLines(Number:Word);
    function SetPaperSize(aSize: Integer; aHeight, aWidth: Single): Boolean;
    {Added Gets for the font information making it easier to use SetFontInformation.}
    property FontName: string read CurrentFontName;
    property FontSize: Word read CurrentFontSize;
    property FontStyle: TFontStyles read CurrentFontStyle;
    property PaperHeight: Single read PaperHeightInches;
    property PaperWidth: Single read PaperWidthInches;
    property Running: Boolean read GetRunning write SetRunning stored True;
    property TheYPosition: Single read LastYPosition write SetYPosition;
    property CharPerLine: Integer read GetColumnsPerLine;
    property LineHeightInches: Single read GetLineHeightInches;
    property LineHeightPixels: Word read GetLineHeightPixels;
    property LinesInDetailArea: Word read GetLinesInDetailArea;
    property LinesLeft: Word read GetLinesLeft;
    property LinesPerPage: Integer read GetLinesPerPage;
    property PageNumber:Integer read GetPageNumber;
    {Added Published properties for ease of setting certain properties at design time.}
  published
    {Properties:
      AutomaticPaging: can be used to automatically do NewPage when printing.
      Title: this text becomes the title of the print job in printer manager
      Preview: whether to print directly to printer of show preview of job
      ShowProgress: display a progress bar or not
      ShowPrintDialog: whether to show print dialog to allow user to select
                       printer, pages to print, etc.
      Orientation: set page orientation (can only be set for all pages)
      ProgressBarMax: sets the maximum progress bar value
      ProgressBarInc: sets the increment for the progress bar
      Transparent: if you want a background image, this sets the text to print
                   transparently so it shows through
      StartPercent: sets the initial zoom percentage for preview
      PaperSize: sets papersize to the printer, letter or legal
      ShowCancel: enables cancel button on the progress bar form
      HelpContext: sets help context for Preview Form
    }
    property AutomaticPaging: Boolean read AutoPaging write AutoPaging
             default False;
    property Title: string read DocumentTitle write DocumentTitle;
    property Preview: Boolean read ActualPreview write ActualPreview
             default True;
    property ShowProgress: Boolean read UseProgress write UseProgress
             default True;
    property ShowPrintDialog: Boolean read ShowDialog write ShowDialog
             default True;
    property Orientation: TPrinterOrientation read PageOrientation
             write PageOrientation default poPortrait;
    property ProgressBarMax: Word read ProgressMax write ProgressMax
             default 100;
    property ProgressBarInc: Word read ProgressInc write ProgressInc
             default 1;
    property Transparent: Boolean read PrintTransparent write PrintTransparent
             default True;
    property StartPercent: Percentages read PreviewPercent write PreviewPercent
             default P2_Fifty;
    property PaperSize: PaperSizes read PaperDimension write PaperDimension
             default PtrSU;
    property ShowCancel: Boolean read GetShowCancel write SetShowCancel
             default True;
    property HelpContext: THelpContext read HelpCntxt write HelpCntxt
             default 0;
    {Events:
      OnNewPage: event for any printing to occur for every page
      (I use this to put a bitmap on every page like a company logo)
      AfterNewPage: event to print after the page has been advanced.
      (I use this for things to occur on all but the first page like
      a continuation header or something)
      AfterPrinterChange: event for change in printer when ShowPrintDialog
      is True.
      OnMarginError: event for SetMargins when margins are in unprintable
      area.  If not assigned, use message to fix or quit.
    }
    property OnNewPage: TNotifyEvent read FPageChange write FPageChange;
    property AfterNewPage: TNotifyEvent read FAPageChange write FAPageChange;
    property AfterPrinterChange: TNotifyEvent read FPrinterChange
                    write FPrinterChange;
    property OnMarginError: TNotifyEvent read FMarginError write FMarginError;
  end;

procedure Register;

implementation

uses Windows;

type
  PrtDef = class(TObject)        { Data portion of the TPrinter object }
    FCanvas: TCanvas;            { Used to access DevMode fields }
    FFonts: TStrings;
    FPageNumber: Integer;
    FPrinters: TStrings;
    FPrinterIndex: Integer;
    FTitle: string;
    FPrinting: Boolean;
    FAborted: Boolean;
    FCapabilities: TPrinterCapabilities;
    State: TPrinterState;
    DC: HDC;
    DevMode: PDeviceMode;
    DeviceMode: THandle;
    FPrinterHandle: THandle;
  end;

constructor TPrintPreview.Create(AOwner: TComponent);
{Create TPrintPreview and set default values}
begin
  inherited Create(AOwner);
  AutomaticPaging := False;
  Title := '';
  Preview := True;
  ShowProgress := True;
  ShowPrintDialog := True;
  Orientation := poPortrait;
  ProgressBarMax := 100;
  ProgressBarInc := 1;
  Transparent := True;
  StartPercent := P2_Fifty;
  PaperSize := PtrSU;
  ShowCancel := True;
  HelpContext := 0;
  PreviewMode := True;
  ScaleByValue := 1.0;
  CurrentFontRecord := 0;
  OnNewPage := nil;
  AfterNewPage := nil;
  OnMarginError := nil;
  AfterPrinterChange := nil;
end;  { TPrintPreview.Create }

destructor TPrintPreview.Destroy;
{ Destroy TPrintPreview and call TComponent destructor}
{ If printing was performed, the other objects created for it
  should have already been destroyed after the print job.}
begin
  OnNewPage := nil;
  AfterNewPage := nil;
  OnMarginError := nil;
  AfterPrinterChange := nil;
  inherited Destroy;
end;  { TPrintPreview.Destroy }

procedure TPrintPreview._DrawBox(XTop,YTop,XBottom,YBottom:Word;
                                 LineWidth: Single; Shading:Word);
   { The low level routine which actually draws the box and shades it as
     desired. The paramaters are in pixels and not inches. }
var
  tmpColor: TColor;
  tmpWidth: Integer;
  tmpStyle: TBrushStyle;
  tmpX,tmpY: Integer;
begin
  with TheCanvas do begin
    tmpX := PenPos.X;
    tmpY := PenPos.Y;
    tmpWidth := Pen.Width;
    Pen.Width := Round(LineWidth * ResolutionY / 72);
    if Pen.Width < 1 then
      Pen.Width := 1;
    {fi}
    tmpColor := Brush.Color;
    Brush.Color := RGB(Shading,Shading,Shading);
    tmpStyle := Brush.Style;
    if Shading = 255 then
      Brush.Style := bsClear
    else
      Brush.Style := bsSolid;
    {fi}
    if not DryRun then
      try
        Rectangle(XTop,YTop,XBottom,YBottom);
      except
        on E: EPrinter do PrinterError(E.Message);
      else
        PrinterError('Unknown failure');
      end;
    {fi}
    Pen.Width := tmpWidth;
    Brush.Color := tmpColor;
    Brush.Style := tmpStyle;
    MoveTo(tmpX,tmpY);
    end;
  {od with Thecanvas}
end;  { TPrintPreview._DrawBox }

procedure TPrintPreview._Write(Alignment: Integer; X: Single; Y: Single;
                               Text:string; Style:TFontStyles; hfFlag: Boolean);
var
  aStyle: TFontStyles;
  XPixels: Integer;
  YPixels: Integer;
  Temp: Integer;
  Buffer: string;
{func/proc}
  function DecimalLength(Text: string): Integer;
  var
    aPos: Integer;
    sTemp: string;
  begin
    aPos := Pos('.',Text);
    if aPos = 0 then
      sTemp := Text
    else
      sTemp := Copy(Text,1,aPos-1);
    {fi}
    Result := GetTextWidthPixels(sTemp);
  end;  { DecimalLength }
begin  { _Write }
   { How many pixels are there in the inches represented by 'Y'? }
  YPixels := 0;
  if Y > -0.01 then begin
    { Printing will occur at an absolute location from the top of the
     page. }
    YPixels := InchesToPixelsY(Y);
    Temp := Trunc(OffsetTop * ScaleByValue);
    if YPixels < Temp then
      YPixels := Temp;
    {fi}
    Temp := Trunc(PaperHeightPixels * ScaleByValue);
    if YPixels > Temp - OffsetBottom then
      YPixels := Temp - OffsetBottom;
    {fi}
    LastYPosition := Y;
    end
  else if Y = -1 then begin
    { Write the text at the next line }
    if DryRun and AutoPaging then begin
      { If the next line we're going to write to exceeds beyond the
         bottom of the detail section, issue a new page }
       if LastYPosition + 2 * GetLineHeightInches >
             PaperHeight - DetailBottom then
         NewPage;
       {fi}
       end;
    {fi}
    YPixels := InchesToPixelsY(LastYPosition + GetLineHeightInches);
    LastYPosition := LastYPosition + GetLineHeightInches;
    end
  else if Y = -2 then begin
   { Write the text on the current line }
    if DryRun and AutoPaging then begin
      if (LastYPosition > ((PaperHeight - DetailBottom) / ScaleByValue)) then
        NewPage;
      {fi}
      end;
    {fi}
    YPixels := InchesToPixelsY(LastYPosition);
    end;
  {fi}
  XPixels := InchesToPixelsX(X);
  case Alignment of
    wtLeft: ;
    wtCenter: Dec(XPixels,Trunc(GetTextWidthPixels(Text)*ScaleByValue) div 2);
    wtRight: Dec(XPixels,Trunc(GetTextWidthPixels(Text)*ScaleByValue));
    wtIndent: ;
    wtDecimal: Dec(XPixels,DecimalLength(Text));
  end{case};
  if (XPixels < OffsetLeft) then
    XPixels := OffsetLeft;
  {fi}
  if PrintTransparent then
    TheCanvas.Brush.Style := bsClear
  else
    TheCanvas.Brush.Style := bsSolid;
  {fi}
  if Style <> [] then begin
    aStyle := TheCanvas.Font.Style;
    TheCanvas.Font.Style := TheCanvas.Font.Style + Style;
    end;
  {fi}
  LastXPosition := PixelsToInchesX(Trunc(XPixels/ScaleByValue) +
                                   GetTextWidthPixels(Text));
  if not DryRun then begin
    if hfFlag then begin   { True when header or footer contains page number}
      try
        if PreviewMode then
          Buffer := Format(Text,[PreviewForm.GetPageNumber,TotalPages])
        else
          Buffer := Format(Text,[Printer.PageNumber,TotalPages]);
        {fi}
      except
        Buffer := Text;
      end; {try}
      end {if then}
    else
      Buffer := Text;
    {fi}
    try
      TheCanvas.TextOut(XPixels-OffsetLeft, YPixels-OffsetTop, Buffer);
    except
      on E: EPrinter do PrinterError(E.Message);
    else
      PrinterError('Unknown failure');
    end;
    LastXPosition := PixelsToInchesX(Trunc(XPixels/ScaleByValue) +
                                     GetTextWidthPixels(Text));
    end
  else
    Pages.StoreText(Alignment,X,LastYPosition,Text,Style,
                    TheCanvas.Brush.Color,hfFlag);
  {fi}
  if Style <> [] then
    TheCanvas.Font.Style := aStyle;
  {fi}
end;  { TPrintPreview._Write }

procedure TPrintPreview.WriteLine(Alignment: Word; Y:Single; Text:string);
begin
  DoWrite(Alignment,Y,Text,False);
end;  { TPrintPreview.WriteLine }

procedure TPrintPreview.Abort;
{Used to abort the print job in progress}
begin
  if ProgressBar <> nil then begin
    ProgressBar.Close;
    ProgressBar.Free;
    ProgressBar := nil;
    end;
  {fi}
  if PreviewMode then
    PreviewForm.Abort
  else
    Printer.Abort;
  Pages.Free;
  Cleanup;
end;  { TPrintPreview.Abort }

procedure TPrintPreview.AfterPageChange;
begin
  if assigned(FAPageChange) then
    FAPageChange(self);
  {fi}
end;  { TPrintPreview.AfterPageChange }

function TPrintPreview.CalculateLineHeight: Integer;
   { Calculate the height of a line plus the normal amount of space between
     each line }
begin
  Result := Trunc(TheCanvas.TextHeight('A') / ScaleByValue);
end;  { TPrintPreview.CalculateLineHeight }

procedure TPrintPreview.CalculateMeasurements;
   { Calculate some necessary measurements. }
begin
   { Call the Windows API function GetTextMetrics() to get the specifics
     of the particular font. }
     GetTextMetrics(TheCanvas.Handle,TextMetrics);

   { Calculate the number of pixels per inch vertical and horizontal.
     'GetDeviceCaps' is a Windows API call. }
  ResolutionY := GetDeviceCaps(TheCanvas.Handle,LogPixelsY);
  ResolutionX := GetDeviceCaps(TheCanvas.Handle,LogPixelsX);

   { Get the offset on the left and top. }
  if PreviewMode then begin
    OffsetLeft := 0;
    OffsetTop := 0;
    end
  else begin
    OffsetLeft := GetDeviceCaps(TheCanvas.Handle,PhysicalOffsetX);
    OffsetTop := GetDeviceCaps(Thecanvas.handle,PhysicalOffsetY);
    end;
  {fi}
   {If printing to a TImage, the size doesn't come back as expected so we get the
    size of the preview image}
  if PreviewMode then begin
    PaperWidthPixels := PreviewForm.GetWidth;
    PaperHeightPixels := PreviewForm.GetHeight;
    end
  else begin
    PaperWidthPixels := GetDeviceCaps(Thecanvas.Handle,PhysicalWidth);
    PaperHeightPixels := GetDeviceCaps(TheCanvas.Handle,PhysicalHeight);
    end;
  {fi}
  PaperWidthInches := PaperWidthPixels / ResolutionX;
  PaperHeightInches := PaperHeightPixels / ResolutionY;
   {Need to set these using the scaling value if in preview mode}
  if PreviewMode then begin
    OffsetRight := 0;
    OffsetBottom := 0;
    end
  else begin
    OffsetRight := PaperWidthPixels - OffsetLeft - Printer.PageWidth;
    OffsetBottom := PaperHeightPixels - OffsetTop - Printer.PageHeight;
    end;
  {fi}
end;  { TPrintPreview.CalculateMeasurements }

procedure TPrintPreview.CheckLayout;
begin
  if assigned(FPrinterChange) then
    FPrinterChange(self);
  {fi}
end;  { TPrintPreview.CheckLayout }

procedure TPrintPreview.Cleanup;
var
  x: Integer;
begin
  for x := 1 to HeaderLines do
    Header[x].Text := '';
  {od for}
  for x := 1 to FooterLines do
    Footer[x].Text := '';
  {od for}
  for x := 1 to Columns do begin
    ColumnInformation[x].XPosition := 0.0;
    ColumnInformation[x].Length := 0.0;
    end;
  {od for}
end;  { TPrintPreview.Cleanup }

procedure TPrintPreview.CreateColumn(Number:Word; XPosition:Single; Length:Single);
{Stores the column information. There is a maximum of NumberrColumns columns}
begin
  if (Number > Columns) or (Number < 1) then begin
    ShowMessage('Column value must be between 1 and ' + IntToStr(Columns) + '.');
    exit;
    end;
  {fi}
  if DryRun then
    Pages.StoreColumn(Number,XPosition,Length);
  {i}
  ColumnInformation[Number].XPosition := XPosition;
  ColumnInformation[Number].Length := Length;
end;  { TPrintPreview.CreateColumn }

function TPrintPreview.CurrentFontName: string;
begin
  result := TheCanvas.Font.Name;
end;  { TPrintPreview.CurrentFontName }

function TPrintPreview.CurrentFontSize: Word;
begin
  result := TheCanvas.Font.Size;
end;  { TPrintPreview.CurrentFontSize }

function TPrintPreview.CurrentFontStyle: TFontStyles;
begin
  result := TheCanvas.Font.Style;
end;  { TPrintPreview.CurrentFontStyle }

procedure TPrintPreview.DrawBox(XTop,YTop,XBottom,YBottom:Single;
                                LineWidth:Single);
   { Draw a box at the X,Y coordinates passed in the parameters }
var
  BLinePixels,BColPixels,ELinePixels,EColPixels: Integer;
begin
  if DryRun then
    Pages.StoreBox(XTop,YTop,XBottom,YBottom,LineWidth,255);
  {fi}
  BLinePixels := InchesToPixelsY(YTop) - OffsetTop;
  ELinePixels := InchesToPixelsY(YBottom) - OffsetTop;
  BColPixels := InchesToPixelsX(XTop) - OffsetLeft;
  EColPixels := InchesToPixelsX(XBottom) - OffsetLeft;
  _DrawBox(BColPixels,BLinePixels,EColPixels,ELinePixels,LineWidth,255);
end;  { TPrintPreview.DrawBox }

procedure TPrintPreview.DrawBoxShaded(XTop,YTop,XBottom,YBottom:Single;
                                      LineWidth:Single; Shading:Word);
   { Draw a box at the X,Y coordinates passed in the parameters }
var
  BLinePixels,BColPixels,ELinePixels,EColPixels: Integer;
begin
  if DryRun then
    Pages.StoreBox(XTop,YTop,XBottom,YBottom,LineWidth,Shading);
  {fi}
  BLinePixels := InchesToPixelsY(YTop) - OffsetTop;
  ELinePixels := InchesToPixelsY(YBottom) - OffsetTop;
  BColPixels := InchesToPixelsX(XTop) - OffsetLeft;
  EColPixels := InchesToPixelsX(XBottom) - OffsetLeft;
  _DrawBox(BColPixels,BLinePixels,EColPixels,ELinePixels,LineWidth,Shading);
end;  { TPrintPreview.DrawBoxShaded }

procedure TPrintPreview.DrawLine(TopX,TopY,BottomX,BottomY:Single; LineWidth:Single);
   { Draw a line beginning at a particular X,Y coordinate and ending at a
     particular X,Y coordinate. }
var
  TopXPixels, BottomXPixels, TopYPixels, BottomYPixels: Integer;
  oldWidth: Integer;

begin
  if DryRun then
    Pages.StoreLine(TopX,TopY,BottomX,BottomY,LineWidth);
  {fi}
  TopXPixels := InchesToPixelsX(TopX);
  BottomXPixels := InchesToPixelsX(BottomX);
  TopYPixels := InchesToPixelsY(TopY);
  BottomYPixels := InchesToPixelsY(BottomY);
  Dec(TopXPixels,OffsetLeft);
  Dec(BottomXPixels,OffsetLeft);
  Dec(TopYPixels,OffsetTop);
  Dec(BottomYPixels,OffsetTop);
  oldWidth := TheCanvas.Pen.Width;
  TheCanvas.Pen.Width := Round(LineWidth * ResolutionY / 72);
  if TheCanvas.Pen.Width < 1 then
    TheCanvas.Pen.Width := 1;
  {fi}
  TheCanvas.MoveTo(TopXPixels,TopYPixels);
  if not DryRun then
    TheCanvas.LineTo(BottomXPixels,BottomYPixels);
  {fi}
  TheCanvas.Pen.Width := oldWidth;
end;  { TPrintPreview.DrawLine }

procedure TPrintPreview.DrawTextBox(XTop,YTop,XBottom,YBottom:Single;
                                    LineWidth:Single; Shading:Word; Text: string;
                                    Column: Integer; Alignment: Word);
{Routine to draw a shaded box with text printed inside. Added this to make
 it easier than drawing the box and then writing the text.
 Alignment: 0:Left 1:Center 2:Right}
var
  BLinePixels,BColPixels,ELinePixels,EColPixels: Integer;
  ColorTmp: TColor;
  WidthTmp: Integer;
  YTemp: Single;
begin
  if DryRun then
    Pages.StoreBox(XTop,YTop,XBottom,YBottom,LineWidth,Shading);
  {fi}
  SaveCurrentFont;
  SetFontInformation(FontName,FontSize,[]);
  BLinePixels := InchesToPixelsY(YTop) - OffsetTop;
  ELinePixels := InchesToPixelsY(YBottom) - OffsetTop;
  BColPixels := InchesToPixelsX(XTop) - OffsetLeft;
  EColPixels := InchesToPixelsX(XBottom) - OffsetLeft;
  WidthTmp := TheCanvas.Pen.Width;
  _DrawBox(BColPixels,BLinePixels,EColPixels,ELinePixels,LineWidth,Shading);
  RestoreCurrentFont;
  ColorTmp := TheCanvas.Brush.Color;
  TheCanvas.Brush.Color := RGB(Shading,Shading,Shading);
  YTemp := (YTop + YBottom - GetLineHeightInches) / 2;
  WriteColumn(Column,Alignment,YTemp,Text);
  TheCanvas.Brush.Color := ColorTmp;
  TheCanvas.Pen.Width := WidthTmp;
end;  { TPrintPreview.DrawTextBox }

procedure TPrintPreview.Finish;
   { 'Finish' must always be called when printing is completed. If preview
     mode is set, then it displays the preview form. }
begin
  try
    WriteHeader;
    WriteFooter;
    OnPageChange;
  except
    ShowMessage('Error in completing last page.');
  end;
  DryRun := false;
  try
    if UseProgress then begin
      ProgressBar.Close;
      ProgressBar.Free;
      ProgressBar := nil;
      end;
    {fi}
    Pages.FromPage := 1;
    Pages.ToPage := Pages.Count;
  except
    ShowMessage('Error preparing final report.');
  end;
    if Running then begin
      if ActualPreview then
        ShowPreview
      else
        PrintFromPreview;
      {fi}
      end;
    {fi}
  try
    Pages.Free;
    Cleanup;
  except
    ShowMessage('Error cleaning up report pages.');
  end;
end;  { TPrintPreview.Finish }

procedure TPrintPreview.FixMargins(var Top,Bottom,Left,Right:Single);
var
  inLeft,inRight,inTop,inBottom,inHeight,inWidth: Single;
begin
  GetPaperParms(inTop,inBottom,inLeft,inRight,inHeight,inWidth);
  if (Left + Right > inWidth) then begin
    Left := inLeft;
    Right := inRight;
    end;
  {fi}
  if (Left < inLeft) then
    Left := inLeft;
  {fi}
  if (Right < inRight) then
    Right := inRight;
  {fi}
   { If the sum of the top and bottom margins exceeds the height of the
     page, set the top margin to the value of 'OffsetTop' and set the
     bottom margin to the value of 'OffsetBottom' }
  if (Top + Bottom >= inHeight) then begin
    Top := inTop;
    Bottom := inBottom;
    end;
  {fi}
  if (Top < inTop) then
    Top := inTop;
  {fi}
  if (Bottom < inBottom) then
    Bottom := inBottom;
  {fi}
end;  { TPrintPreview.FixMargins }

function TPrintPreview.GetColumnLeft(Number:Word): Single;
{Function to return the left X coordinate of the desired column.}
begin
  Result := ColumnInformation[Number].XPosition;
end;  { TPrintPreview.GetColumnLeft }

function TPrintPreview.GetColumnRight(Number:Word): Single;
{Function to return the Right X coordinate of the desired column.}
begin
  Result := ColumnInformation[Number].XPosition + ColumnInformation[Number].Length;
end;  { TPrintPreview.GetColumnRight }

function TPrintPreview.GetColumnsPerLine: Integer;
   { How many columns are there in a Line? }
var
  Pixels: Integer;
begin
  Pixels := PaperWidthPixels - LeftMargin - RightMargin;
  Result := Pixels div TheCanvas.TextWidth('B');
end;  { TPrintPreview.GetColumnsPerLine }

function TPrintPreview.GetLineHeightInches: Single;
{ Returns the height of current line in inches }
begin
  Result := PixelsToInchesY(GetLineHeightPixels);
end;  { TPrintPreview.GetLineHeightInches }

function TPrintPreview.GetLineHeightPixels: Word;
{ Returns the height of current line in pixels }
begin
  Result := CalculateLineHeight;
end;  { TPrintPreview.GetLineHeightPixels }

function TPrintPreview.GetLinesInDetailArea: Word;
   { Return the number of lines in the detail area }
begin
  Result := InchesToPixelsY(PaperHeight - DetailBottom - DetailTop) div CalculateLineHeight;
end;  { TPrintPreview.GetLinesInDetailArea }

function TPrintPreview.GetLinesLeft: Word;
   { Return the number of lines left in the detail area }
var
  Lines: Single;
begin
  Lines := (PaperHeight - DetailBottom - LastYPosition) / GetLineHeightInches;
  Result := Trunc(Lines);
  if Result >= 1 then Dec(Result);
end;  { TPrintPreview.GetLinesLeft }

function TPrintPreview.GetLinesPerPage: Integer;
   { Return the number of lines on the entire page }
begin
  Result := (PaperHeightPixels - TopMargin - BottomMargin) div CalculateLineHeight;
end;  { TPrintPreview.GetLinesPerPage }

procedure TPrintPreview.GetMargins(var Top,Bottom,Left,Right:Single);
begin
  Left := inLeftMargin;
  Right := inRightMargin;
  Top := inTopMargin;
  Bottom := inBottommargin;
end;  { TPrintPreview.getMargins }

function TPrintPreview.GetPageNumber;
   { Return the current page number }
begin
  if PreviewMode then
    Result := PreviewForm.GetPageNumber
  else
    Result := Printer.PageNumber;
  {fi}
end;  { TPrintPreview.GetPageNumber }

procedure TPrintPreview.GetPaperParms(var Top,Bottom,Left,Right,
               Height,Width: Single);
var
  iTop, iLeft, iHeight, iWidth: Integer;
  ResX, ResY: Integer;
begin
  ResX := GetDeviceCaps(Printer.Handle,LogPixelsX);
  ResY := GetDeviceCaps(Printer.Handle,LogPixelsY);
  iHeight := GetDeviceCaps(Printer.Handle,PhysicalHeight);
  Height := iHeight/ResY;
  iWidth := GetDeviceCaps(Printer.Handle,PhysicalWidth);
  Width := iWidth/ResX;
  iLeft := GetDeviceCaps(Printer.Handle,PhysicalOffsetX);
  Left := iLeft/ResX;
  iTop := GetDeviceCaps(Printer.Handle,PhysicalOffsetY);
  Top := iTop/ResY;
  Right := (iWidth - iLeft - Printer.PageWidth)/ResX;
  Bottom := (iHeight - iTop - Printer.PageHeight)/ResY;
end;  { TPrintPreview.GetPaperParms }

procedure TPrintPreview.GetPixelsPerInch(var X:Word; var Y:Word);
{ Returns the current pixels per inch of the canvas }
begin
  X := ResolutionX;
  Y := ResolutionY;
end;  { TPrintPreview.GetPixelsPerInch }

procedure TPrintPreview.GetPixelsPerPage(var X:Word; var Y:Word);
{ Returns the number of pixels across the printable area of the page }
begin
  X := PaperWidthPixels - OffsetLeft - OffsetRight;
  Y := PaperHeightPixels - OffsetTop - OffsetBottom;
end;  { TPrintPreview.GetPixelsPerPage }

function TPrintPreview.GetRunning: Boolean;
begin
  Result := RunningFlag;
end;  { TPrintPreview.GetRunning }

function TPrintPreview.GetScale: Single;
{Returns to current scale multiplier}
begin
  result := ScaleByValue;
end;  { TPrintPreview.GetScale }

function TPrintPreview.GetShowCancel: Boolean;
begin
  Result := ShowCancelFlag;
end;  { TPrintPreview.GetShowCancel }

function TPrintPreview.GetTextWidthPixels(Text:string): Integer;
   { Return the width of the text contained in 'Text' in pixels }
begin
  Result := Round(0.5 + TheCanvas.TextWidth(Text) / ScaleByValue);
end;  { TPrintPreview.GetTextWidthPixels }

function TPrintPreview.GetTextWidthInches(Text:string): Single;
   { Return the width of the text contained in 'Text' in inches }
begin
  Result := PixelsToInchesX(GetTextWidthPixels(Text));
end;  { TPrintPreview.GetTextWidthInches }

procedure TPrintPreview.IncBar(x: Integer);
{ Increments the progress bar value by X }
begin
  if UseProgress then
    ProgressBar.IncBar(x);
  {fi}
end;  { TPrintPreview.IncBar }

function TPrintPreview.InchesToPixelsX(Inches: Single): Integer;
   { Convert the horizontal inches represented in 'Inches' to pixels }
var
  Value: Single;
begin
  Value := Inches * ResolutionX * ScaleByValue;
  Result := Trunc(Value);
end;  { TPrintPreview.InchesToPixelsX }

function TPrintPreview.InchesToPixelsY(Inches: Single): Integer;
   { Convert the vertical inches represented in 'Inches' to pixels }
var
  Value: Single;
begin
  Value := Inches * ResolutionY * ScaleByValue;
  Result := Trunc(Value);
end;  { TPrintPreview.InchesToPixelsY }

procedure TPrintPreview.Increment;
{ Increment the progress bar with the value of ProgressInc }
begin
  if UseProgress then
    ProgressBar.IncBar(ProgressInc);
  {fi}
end;  { TPrintPreview.Increment }

function TPrintPreview.LinesToPixels(Line:Integer): Integer;
   { Calculate the number of vertical pixels in 'Line' }
begin
  if (Line <= 0) then
    Line := 1;
  {fi}
  Result := (Line-1) * CalculateLineHeight;
end;  { TPrintPreview.LinesToPixels }

procedure TPrintPreview.NewLines(Number:Word);
   { Generate the number of line feeds represented in 'Number' }
var
  I: Word;
begin
  for I := 1 to Number do
    NextLine;
  {od for}
end;  { TPrintPreview.NewLines }

procedure TPrintPreview.NewPage;
   { Issue a new page }
var
  oldAuto: Boolean;
begin
  WriteHeader;
  WriteFooter;
  oldAuto := AutomaticPaging;
  AutomaticPaging := false;
  OnPageChange;
  AutomaticPaging := oldAuto;
  if DryRun then begin
    Pages.NewPage;
    SetFontInformation(CurrentFontName,CurrentFontSize,CurrentFontStyle);
    inc(TotalPages);
    LastYPosition := DetailTop - GetLineHeightInches;
    LastXPosition := inLeftMargin;
    end
  else if not PreviewMode then begin
    try
      Printer.NewPage;
    except
      on E: EPrinter do PrinterError(E.Message);
    else
      PrinterError('Unknown failure');
    end;
    LastYPosition := DetailTop - GetLineHeightInches;
    LastXPosition := inLeftMargin;
    end;
  {fi}
  SetTopOfPage;
  oldAuto := AutomaticPaging;
  AutomaticPaging := false;
  AfterPageChange;
  AutomaticPaging := oldAuto;
end;  { TPrintPreview.NewPage }

procedure TPrintPreview.NextLine;
{ Advances pen pos to next line on canvas }
var
  OneLine: Single;
begin
  if DryRun then
    Pages.StoreGeneral(commNewLine);
  {fi}
  OneLine := GetLineHeightInches;
  if LastYPosition +  OneLine >
    PaperHeight - DetailBottom / ScaleByValue then
    NewPage
  else
    LastYPosition := LastYPosition + OneLine;
  {fi}
  LastXPosition := inLeftMargin;
end;  { TPrintPreview.NextLine }

procedure TPrintPreview.OnPageChange;
begin
  if assigned(FPageChange) then
    FPageChange(self);
  {fi}
end;  { TPrintPreview.OnPageChange }

function TPrintPreview.PixelsToInchesX(Pixels: Integer): Single;
{ Converts "Pixels" to inches using horizontal resolution }
begin
  Result := Pixels / ResolutionX;
end;  { TPrintPreview.PixelsToInchesX }

function TPrintPreview.PixelsToInchesY(Pixels: Integer): Single;
{ Converts "Pixels" to inches using vertical resolution }
begin
  Result := Pixels / ResolutionY;
end;  { TPrintPreview.PixelsToInchesY }

procedure TPrintPreview.PrinterError(Msg: string);
begin
  MessageDlg(Msg,mtError,[mbOk],0);
  Abort;
end;  { TPrintPreview.PrinterError }

procedure TPrintPreview.PrintFromPreview;
{ Used to print the actual commands that were stored either from the preview
  form or directly when Finish is called to end the print job. }
var
  x: Integer;
  xrInc, xrCurrent: Single;
  oldCanvas: TCanvas;
begin
  if ShowDialog then
    if not Pages.SetPrintOptions then
      exit
    else if Pages.CheckPrinter then
      CheckLayout;
    {fi}
  {fi}
  if not Running then  { If CheckLayout sets Running False then no printing }
    Exit;
  {fi}
  PreviewMode := false;
  xrInc := 100 / ((Pages.ToPage + 1) - Pages.FromPage);
  xrCurrent := 0.0;
  oldCanvas := TheCanvas;
  Printer.Title := DocumentTitle;
  if PreviewMode then
    PreviewForm.SetOrient(PageOrientation)
  else
    Printer.Orientation := PageOrientation;
  {fi}
  try
    Printer.BeginDoc;
  except
    on E: EPrinter do PrinterError(E.Message);
  else
    PrinterError('Unknown failure');
  end;
  TheCanvas := Printer.Canvas;
  CalculateMeasurements;
  if UseProgress then begin
    ProgressBar := TPrintBarForm.Create(Application);
    with ProgressBar do  begin
      SetDisplayText('Printing report............');
      Left := Application.MainForm.Left + (Application.MainForm.Width - Width) div 2;
      Top := Application.MainForm.Top + (Application.MainForm.Height - Height) div 2;
      Show;
      UpdateBar(0);
      end;
    {od with}
    end;
  {fi}
  for x := Pages.FromPage to Pages.ToPage do
    if Running then begin
      PrintPage(Pages.Get(x),x);
      if x < Pages.ToPage then begin
        try
          Printer.NewPage;
        except
          on E: EPrinter do PrinterError(E.Message);
        else
          PrinterError('Unknown failure');
        end;
        SetTopOfPage;
        end;
      {fi}
      xrCurrent := xrCurrent + xrInc;
      if UseProgress then
        ProgressBar.UpdateBar(Round(xrCurrent));
      {fi}
      end
    else
      Break;
    {fi}
  {od for}
  if UseProgress and Running then
    ProgressBar.UpdateBar(100);
  {fi}
  if Running then
    try
      Printer.EndDoc;
    except
      on E: EPrinter do PrinterError(E.Message);
    else
      PrinterError('Unknown failure');
    end
  else if Printer.Printing then
    try
      Printer.Abort;
    except
      on E: EPrinter do PrinterError(E.Message);
    else
      PrinterError('Unknown failure');
    end;
  {fi}
  PreviewMode := true;
  TheCanvas := oldCanvas;
  if UseProgress then begin
    ProgressBar.Close;
    ProgressBar.Free;
    ProgressBar := nil;
    end;
  {fi}
end;  { TPrintPreview.PrintFromPreview }

procedure TPrintPreview.PrintGraphic(X,Y,W,H: Single;thePicture: TImage);
{ Used to copy contents of TImage to printing canvas }
var
  x1,x2,y1,y2: Integer;
begin
  if DryRun then begin
    Pages.StoreGraphic(X,Y,H,W,thePicture);
    exit;
    end;
  {fi}
  x1 := Max(InchesToPixelsX(X)-OffsetLeft,0);
  x2 := InchesToPixelsX(W);
  y1 := Max(InchesToPixelsY(Y)-OffsetTop,0);
  y2 := InchesToPixelsY(H);
  TheCanvas.StretchDraw(rect(x1,y1,x1+x2,y1+y2),thePicture.Picture.Graphic);
end;  { TPrintPreview.PrintGraphic }

procedure TPrintPreview.PrintPage(Page: TMWPageList;iPage: Integer);
{ This procedure will loop through a given pages command list and output that
  information to the current printing canvas }
var
  x: Integer;
  oldColor: TColor;
  tCommand: TMWPrintCommand;
  SkipNext: Integer;
begin
  SkipNext := 0;
  SetTopOfPage;
  for x := 1 to Page.Count do begin
    if SkipNext = 0 then begin
      tCommand := Page.Get(x);
      case tCommand.Command of
        commBox:
          try
            with tCommand as TMWBox do
              DrawBoxShaded(X1,Y1,X2,Y2,LineWidth,Shading);
            {od with}
          except
            ShowMessage('Error in Box Drawing command.');
          end;
        commLine:
          try
            with tCommand as TMWLine do
              DrawLine(X1,Y1,X2,Y2,LineWidth);
            {od with}
          except
            ShowMessage('Error in Line Drawing command.');
          end;
        commNewLine:
          try
            NextLine;
          except
            ShowMessage('Error in Next Line command.');
          end;
        commGraphic:
          try
            with tCommand as TMWGraphic do
              PrintGraphic(X,Y,Height,Width,Bitmap);
            {od with}
          except
            ShowMessage('Error in Graphics Drawing routine.');
          end;
        commDetailArea:
          try
            with tCommand as TMWDetail do
              SetDetailTopBottom(X,Y);
            {od with}
          except
            ShowMessage('Error in Setting Detail Top/Bottom settings.');
          end;
        commFont:
          try
            with tCommand as TMWFont do
              SetFontInformation(Name,Size,Style);
            {od with}
          except
            ShowMessage('Error in Changing Font information');
          end;
        commFontStyle:
          try
            with tCommand as TMWFontStyle do
              SetFontStyle(Style);
            {od with}
          except
            ShowMessage('Error in Changing Font attribute');
          end;
        commFooterDim:
          try
            with tCommand as TMWFooterDim do
              SetFooterDimensions(X1,Y1,X2,Y2,Boxed,LineWidth,Shading);
            {od with}
          except
            ShowMessage('Error in Setting Footer Dimensions.');
          end;
        commFooterInf:
          try
            with tCommand as TMWFooterInf do
              SetFooterInformation(Line,Alignment,yPosition,Text,FontName,Size,Style);
            {od with}
          except
            ShowMessage('Error in Setting Footer Information.');
          end;
        commHeaderDim:
          try
            with tCommand as TMWHeaderDim do
              SetHeaderDimensions(X1,Y1,X2,Y2,Boxed,LineWidth,Shading);
            {od with}
          except
            ShowMessage('Error in Setting Header Dimensions.');
          end;
        commHeaderInf:
          try
            with tCommand as TMWHeaderInf do
              SetHeaderInformation(Line,Alignment,yPosition,Text,FontName,Size,Style);
          {od with}
          except
          ShowMessage('Error in Setting Header Information.');
          end;
        commMargins:
          try
          with tCommand as TMWMargins do
            SetMargins(X1,Y1,X2,Y2);
          {od with}
          except
            ShowMessage('Error in Setting Margins.');
          end;
        commOrient:
          try
            with tCommand as TMWOrient do
             PageOrientation := Orientation;
            {od with}
          except
            ShowMessage('Error in Setting Page Orientation.');
          end;
        commWritePage:
          try
            tCommand := Page.Get(x+1);
            with tCommand as TMWFont do
              SetFontInformation(Name,Size,Style);
            {od with}
            inc(SkipNext);
            tCommand := Page.Get(x+2);
            if tCommand.Command = commTab then begin
              with tCommand as TMWTab do
                SetTab(Tab);
              {od with}
              inc(SkipNext);
              end;
            {fi}
            inc(SkipNext);
          except
            ShowMessage('Error writing page number information.');
          end;
        commTab:
          try
            with tCommand as TMWTab do
              SetTab(Tab);
            {od with}
          except
           ShowMessage('Error Setting Tab.');
          end;
        commTopOfPage:
          try
            SetTopOfPage;
          except
            ShowMessage('Error Setting Top of Page.');
          end;
        commSetY:
          try
            with tCommand as TMWSetY do
              LastYPosition := yPosition;
            {od with}
          except
            ShowMessage('Error Setting Vertical Position.');
          end;
        commText:
          try
            with tCommand as TMWText do begin
              oldColor := TheCanvas.Brush.Color;
              TheCanvas.Brush.Color := BrushColor;
              _Write(Alignment,X,Y,Text,Style,hfFlag);
              TheCanvas.Brush.Color := oldColor;
              end;
            {od with}
          except
            ShowMessage('Error Writing Line of Stylized Text.');
          end;
        commColumn:
          try
            with tCommand as TMWColumn do
              CreateColumn(Number,XPosition,Length);
            {od with}
          except
            ShowMessage('Error Storing Column Setting.');
          end;
        {esac}
      end  {Case}
      end {SkipNext = 0}
    else
      dec(SkipNext);
    {fi SkipNext}
    end;
  {od for x := 1 to Page.Count}
end;  { TPrintPreview.PrintPage }

procedure TPrintPreview.RestoreCurrentFont;
{ Restores the font information after being saved }
begin
  if (CurrentFontRecord in [1..10]) then
    with CurrentFonts[CurrentFontRecord] do
      SetFontInformation(Name,Size,Style);
    {od with}
  {fi}
  dec(CurrentFontRecord);
end;  { TPrintPreview.RestoreCurrentFont }

procedure TPrintPreview.SaveCurrentFont;
{ Saves the current font information }
begin
  inc(CurrentFontRecord);
  if CurrentFontRecord in [1..10] then
    with CurrentFonts[CurrentFontRecord] do begin
      Name := TheCanvas.Font.Name;
      Size := TheCanvas.Font.Size;
      Style := TheCanvas.Font.Style;
      end;
    {od with}
  {fi}
end;  { TPrintPreview.SaveCurrentFont }

procedure TPrintPreview.SetDetailTopBottom(Top: Single; Bottom: Single);
{ Sets the top and bottom margins of the detail printing area of the
  report }
begin
  if Top < inTopMargin then
    Top := inTopMargin;
  {fi}
  if Bottom < inBottomMargin then
    Bottom := inBottomMargin;
  {fi}
  if DryRun then
    Pages.StoreDetailArea(Top,Bottom);
  {fi}
  DetailTop := Top;
  DetailBottom := Bottom;
  LastYPosition := Top - GetLineHeightInches;
end;  { TPrintPreview.SetDetailTopBottom }

procedure TPrintPreview.SetFontStyle(Style: TFontStyles);
   { Change the current font information }
begin
  if DryRun then
    Pages.StoreFontStyle(Style);
  {fi}
  TheCanvas.Font.Style := Style;
  CalculateMeasurements;
end;

procedure TPrintPreview.SetFontInformation(Name:string; Size:Word;
             Style: TFontStyles);
   { Change the current font information }
begin
  if DryRun then
    Pages.StoreFontChange(Name,Size,Style);
  {fi}
  TheCanvas.Font.Name := Name;
  TheCanvas.Font.Size := Size;
  TheCanvas.Font.Style := Style;
  TheCanvas.Pen.Mode := pmMask;
  CalculateMeasurements;
end;  { TPrintPreview.SetFontInformation }

procedure TPrintPreview.SetFooterDimensions(XTop,YTop,XBottom,YBottom:Single;
                                            Boxed: Boolean; LineWidth:Single;
                                            Shading:Word);
{ Sets the area of the page footer and whether it should be boxed and shaded }
begin
  if DryRun then
    Pages.StoreFooterDim(XTop,YTop,XBottom,YBottom,Boxed,LineWidth,Shading);
  {fi}
  FooterCoordinates.XTop := XTop;
  FooterCoordinates.XBottom := XBottom;
  FooterCoordinates.YTop := YTop;
  FooterCoordinates.YBottom := YBottom;
  FooterCoordinates.Boxed := Boxed;
  FooterCoordinates.LineWidth := LineWidth;
  FooterCoordinates.Shading := Shading;
end;  { TPrintPreview.SetFooterDimensions }

procedure TPrintPreview.SetFooterInformation(Line:Integer; Alignment:Word;
            YPosition: Single; Text:string; FontName:string; FontSize: Word;
            FontStyle: TFontStyles);
{ Sets the footer information to be printed }
begin
  if (Line > FooterLines) then
    Exit;
  {fi}
  if DryRun then
    Pages.StoreFooterInf(Line,Alignment,YPosition,Text,FontName,FontSize,FontStyle);
  {fi}
  Footer[Line].Text := Text;
  Footer[Line].YPosition := YPosition;
  Footer[Line].Alignment := Alignment;
  Footer[Line].FontName := FontName;
  Footer[Line].FontSize := FontSize;
  Footer[Line].FontStyle := FontStyle;
end;  { TPrintPreview.SetFooterInformation }

procedure TPrintPreview.SetHeaderDimensions(XTop,YTop,XBottom,YBottom:Single;
                                            Boxed: Boolean; LineWidth:Single;
                                            Shading:Word);
{ Sets the area of the header for the page }
begin
  if DryRun then
    Pages.StoreHeaderDim(XTop,YTop,XBottom,YBottom,Boxed,LineWidth,Shading);
  {fi}
  HeaderCoordinates.XTop := XTop;
  HeaderCoordinates.XBottom := XBottom;
  HeaderCoordinates.YTop := YTop;
  HeaderCoordinates.YBottom := YBottom;
  HeaderCoordinates.Boxed := Boxed;
  HeaderCoordinates.LineWidth := LineWidth;
  HeaderCoordinates.Shading := Shading;
end;  { TPrintPreview.SetHeaderDimensions }

procedure TPrintPreview.SetHeaderInformation(Line:Integer; Alignment:Word;
            YPosition: Single; Text:string; FontName:string; FontSize: Word;
            FontStyle: TFontStyles);
{ Sets the header information to be printed }
begin
  if (Line > HeaderLines) then
    Exit;
  {fi}
  if DryRun then
    Pages.StoreHeaderInf(Line,Alignment,YPosition,Text,FontName,FontSize,FontStyle);
  {fi}
  Header[Line].Text := Text;
  Header[Line].YPosition := YPosition;
  Header[Line].Alignment := Alignment;
  Header[Line].FontName := FontName;
  Header[Line].FontSize := FontSize;
  Header[Line].FontStyle := FontStyle;
end;  { TPrintPreview.SetHeaderInformation }

procedure TPrintPreview.SetMargins(Top,Bottom,Left,Right:Single);
   { Set the top, bottom, left and right margins in inches }
var
  Resp: Word;
begin
   { If the sum of the left and right margins exceeds the width of the page,
     set the left margin to the value of 'OffsetLeft' and set the right
     margin to the value of 'OffsetRight' }
  { Calculate offsets in inches }
  if not TestMargins(Top,Bottom,Left,Right) then
    if Assigned(FMarginError) then begin
      FMarginError(self);
      Top := inTopMargin;
      Bottom := inBottomMargin;
      Left := inLeftMargin;
      Right := inRightMargin;
      end
    else begin
    Resp := MessageDlg('One or more margins are set outside the printable ' +
      'area of the page. Do you want fix them?',mtWarning,mbYesNoCancel,0);
    case Resp of
      mrYes: FixMargins(Top,Bottom,Left,Right);
      mrCancel: Running := False;
    end {case};
    end {else};
  {fi}
   { Convert everything to pixels }
  TopMargin := InchesToPixelsY(Top);
  if (TopMargin < OffsetTop) then
    TopMargin := OffsetTop;
  {fi}
  BottomMargin := InchesToPixelsY(Bottom);
  if (BottomMargin < OffsetBottom) then
    BottomMargin := OffsetBottom;
  {fi}
  LeftMargin := InchesToPixelsX(Left);
  if (LeftMargin < OffsetLeft) then
    LeftMargin := OffsetLeft;
  {fi}
  RightMargin := InchesToPixelsX(Right);
  if (RightMargin < OffsetRight) then
    RightMargin := OffsetRight;
  {fi}
  if DryRun then
    Pages.StoreMargins(Top,Bottom,Left,Right);
  {fi}
  inLeftMargin := Left;
  inRightMargin := Right;
  inTopMargin := Top;
  inBottomMargin := Bottom;
  { Make sure that detail area is within top and bottom magins }
  if DryRun and
      ((DetailTop < inTopmargin) or (DetailBottom < inBottomMargin)) then begin
    if DetailTop < inTopMargin then
      DetailTop := inTopMargin;
    {fi}
    if DetailBottom < inBottomMargin then
      DetailBottom := inBottomMargin;
    {fi}
    SetDetailTopBottom(DetailTop,DetailBottom);
    end;
  {fi}
end;  { TPrintPreview.SetMargins }

function TPrintPreview.SetPaperSize(aSize: Integer;
          aHeight, aWidth: Single): Boolean;
{ If aSize = 0, sets the paper to the specified size,
  else, sets to predefined size. }
var
  aDevice: string;
  aDriver: string;
  aPort: string;
  RetLen: Single;
  SaveSize: Word;
begin
  SetLength(ADevice,255);
  SetLength(aDriver,255);
  setLength(aPort,255);
  Printer.GetPrinter(PChar(ADevice),PChar(ADriver),
        PChar(APort),ADeviceMode);
  if aDeviceMode = 0 then
    Result := False
  else begin
    SaveSize := PrtDef(Printer).DevMode^.dmPaperSize;
    if aSize = 0 then begin
      PrtDef(Printer).DevMode^.dmPaperLength := Round(254 * aHeight);
      PrtDef(Printer).DevMode^.dmPaperWidth := Round(254 * aWidth);
      PrtDef(Printer).DevMode^.dmPaperSize := 256;
      PrtDef(Printer).DevMode^.dmFields :=
          (DM_PAPERSIZE+DM_PAPERLENGTH+DM_PAPERWIDTH);
      end
    else begin
      PrtDef(Printer).DevMode^.dmPaperSize := aSize;
      PrtDef(Printer).DevMode^.dmFields := (DM_PAPERSIZE);
      end;
    {fi}
    Printer.SetPrinter(PChar(ADevice),PChar(ADriver),
        PChar(APort),ADeviceMode);
    try
      RetLen := GetDeviceCaps(Printer.Handle,PhysicalHeight)/
                GetDeviceCaps(Printer.Handle,LogPixelsY);
      if RetLen <= 0 then begin
        MessageDlg('Could not set form to '+FloatToStr(aWidth)+'x'+
                   FloatToStr(aHeight),mtError,[mbOk],0);
        SetPaperSize(SaveSize,aHeight,aWidth);
        Result := False;
        end
      else if aSize = 0 then begin
        Result := RetLen = aHeight;
        end
      else
        Result := True;
      {fi}
    except
      MessageDlg('Could not set form to '+FloatToStr(aWidth)+'x'+
                 FloatToStr(aHeight),mtError,[mbOk],0);
      SetPaperSize(SaveSize,aHeight,aWidth);
      Result := False;
    end {try};
    end;
  {fi}
end;  { TPrintPreview.SetPaperSize }

procedure TPrintPreview.SetPrnPrinter(Current: TImage);
{ Sets the current printing canvas of the preview form }
begin
  ThePrinter := Current;
  TheCanvas := ThePrinter.Canvas;
end;  { TPrintPreview.SetPrnPrinter }

procedure TPrintPreview.SetRunning(SetIt: Boolean);
begin
  RunningFlag := SetIt;
end;  { TPrintPreview.SetRunning }

procedure TPrintPreview.SetScale(ScaleFactor: Single);
{Sets the current scale multiplier}
begin
  ScaleByValue := ScaleFactor;
  CalculateMeasurements;
end;  { TPrintPreview.SetScale }

procedure TPrintPreview.SetShowCancel(SetIt: Boolean);
begin
  ShowCancelFlag := SetIt;
end;  { TPrintPreview.SetShowCancel }

procedure TPrintPreview.SetTab(Inches: Single);
begin
  if DryRun then
    Pages.StoreTab(Inches);
  {fi}
  inCurrentTab := Inches;
  CurrentTab := InchesToPixelsX(Inches);
end;  { TPrintPreview.SetTab }

procedure TPrintPreview.SetTopOfPage;
{ Sets the printing canvas to the top of the detail area on the page }
begin
  if DryRun then
    Pages.StoreGeneral(commTopOfPage);
  {fi}
  LastYPosition := DetailTop - GetLineHeightInches;
  LastXPosition := inLeftMargin;
end;  { TPrintPreview.SetTopOfPage }

procedure TPrintPreview.SetYPosition(YPos: Single);
begin
  LastYPosition := YPos;
  if DryRun then
    Pages.StoreSetY(YPos);
  {fi}
end;  { TPrintPreview.SetYPosition }

procedure TPrintPreview.ShowPreview;
{ Displays the preview form }
begin
  If not PreviewMode then
    exit;
  {fi}
  with PreviewForm do begin
    { Set links back to private methods }
    StartPercent := ord(PreviewPercent);
    PrtFromPreview := PrintFromPreview;
    PrtPage := PrintPage;
    SetPrnPtr := SetPrnPrinter;
    GetScl := GetScale;
    SetScl := SetScale;
    { Show the preview form }
    ShowModal;
    end;
  {od with}
end;  { TPrintPreview.ShowPreview }

procedure TPrintPreview.Start;
   { This function MUST be called first before any other printing function }
var
  I: Integer;
  SaveDryRun: Boolean;
begin
{Added this to implement the stored commands procedures. We force the printing to
 run through first in DryRun mode. This gives us the ability to store all print
 commands and then replay them when the Finish command is executed. If the preview
 mode is set, then we need to pass control to the preview form. It will drive the
 page printing through a procedure call hook that we set with OnPrintPage event of
 the preview object.}

  Application.CreateForm(TPreviewForm,PreviewForm);
  if HelpCntxt > 0 then
    PreviewForm.HelpContext := HelpCntxt;
  {fi}
  Case PaperSize of
    Letter:
      SetPaperSize(DMPAPER_LETTER,0.0,0.0);
    Legal:
      SetPaperSize(DMPAPER_LEGAL,0.0,0.0);
    {esac}
  end;
  GetPaperParms(inTopMargin,inBottomMargin,
             inLeftMargin,inRightMargin,tPageHeight,tPageWidth);
  { Set variables without putting a command on pages }
  SaveDryRun := DryRun;
  DryRun := False;
  SetMargins(inTopMargin,inBottomMargin,inLeftMargin,inRightMargin);
  DryRun := SaveDryRun;
  SetPrnPrinter(PreviewForm.GetCurrentPage);
  PreviewForm.SetOrient(PageOrientation);
  PreviewMode := True;
  Pages := TMWPages.Create;
  Pages.NewPage;
  DryRun := true;
  TotalPages := 1;
  ScaleByValue := 1.0;
  CurrentFontRecord := 0;
  CalculateMeasurements;
  if UseProgress then begin
    ProgressBar := TPrintBarForm.Create(Application);
    with ProgressBar do begin
      Left := Application.MainForm.Left + (Application.MainForm.Width - Width) div 2;
      Top := Application.MainForm.Top + (Application.MainForm.Height - Height) div 2;
      SetDisplayText('Preparing report............');
      PrintBar.Max := ProgressMax;
      Show;
      UpdateBar(0);
      end;
    {od with}
    end;
  {fi}
  { Set detail area to current page and font }
  SaveDryRun := DryRun;
  DryRun := False;
  SetDetailTopBottom(inTopMargin,inBottomMargin);
  DryRun := SaveDryRun;
  for I := 1 to HeaderLines do
    Header[I].Text := '';        { Clears header lines }
  {od for}
  HeaderCoordinates.Boxed := False;
  HeaderCoordinates.Shading := 0;
  for I := 1 to FooterLines do
    Footer[I].Text := '';        { Clears footer lines }
  {od for}
  FooterCoordinates.Boxed := False;
  FooterCoordinates.Shading := 0;
  LastYPosition := 0.0;
  LastXPosition := 0.0;
  Running := True;
end;  { TPrintPreview.Start }

function TPrintPreview.TestMargins(Top,Bottom,Left,Right:Single): Boolean;
var
  inLeft,inRight,inTop,inBottom,inHeight,inWidth: Single;
begin
  GetPaperParms(inTop,inBottom,inLeft,inRight,inHeight,inWidth);
  Result := not ((Left + Right > inWidth) or
      (Top + Bottom >= inHeight) or
      (Left < inLeft) or (Right < inRight) or
      (Top < inTop) or (Bottom < inBottom))
end;  { TPrintPreview.TestMargins }

procedure TPrintPreview.UpdateBar(X: Integer);
{ Sets the current progress bar value }
begin
  if UseProgress then
    ProgressBar.UpdateBar(x);
  {fi}
end;  { TPrintPreview.UpdateBar }

procedure TPrintPreview.WriteColumn(ColumnNumber:Word; Alignment: Word;
             Y:Single; Text:string);
   { Write text  against the column represented by 'ColumnInformation[ColumnNumber]' }
var
  X: Single;
begin
  X := 0;
   { How many pixels does the text in 'Text' require? }
  case Alignment of
    0: X := ColumnInformation[ColumnNumber].XPosition;     { Left }
    1: X := ColumnInformation[ColumnNumber].XPosition +    { Center }
            ColumnInformation[ColumnNumber].Length / 2;
    2: X := ColumnInformation[ColumnNumber].XPosition +    { Right }
            ColumnInformation[ColumnNumber].Length;
    {esac}
  end;
  _Write(Alignment,X,Y,Text,[],False);
end;  { TPrintPreview.WriteColumn }

procedure TPrintPreview.WriteDecimal(X:Single; Y:Single; Text:string);
begin
  _Write(wtDecimal,X,Y,Text,[],False);
end; { TPrintPreview.WriteDecimal }

procedure TPrintPreview.WriteFooter;
   { If any footers are defined, write them }
var
  I: Integer;
  Temp: Boolean;
  BotPos: Single;
  aFlag: Boolean;
begin
  if Dryrun then begin
    SaveCurrentFont;
     { Set 'AutoPaging' off.  Otherwise the footer will not get written
       correctly. }
    Temp := AutoPaging;
    AutoPaging := False;
    for I := 1 to FooterLines do begin
      if (Length(Footer[I].Text) > 0) then begin
        with Footer[I] do begin
          SetFontInformation(FontName,FontSize,FontStyle);
          BotPos := PaperHeightInches - inBottomMargin - GetLineHeightInches;
          aFlag := (Pos('%u',Text) > 0);
          DoWrite(Alignment,BotPos - YPosition,Text,aFlag);
          end;
        {od with}
        end;
      {fi}
      end;
    {od for}
    RestoreCurrentFont;
     { Does the user desire a box around the footer? }
    if (FooterCoordinates.Boxed = True) then begin
      BotPos := PaperHeightInches - inBottomMargin;
      if (FooterCoordinates.Shading > 0) then
        DrawBoxShaded(FooterCoordinates.XTop,BotPos - FooterCoordinates.YTop,
                       FooterCoordinates.XBottom,BotPos - FooterCoordinates.YBottom,
                       FooterCoordinates.LineWidth,FooterCoordinates.Shading)
      else
        DrawBox(FooterCoordinates.XTop,BotPos - FooterCoordinates.YTop,
                 FooterCoordinates.XBottom,BotPos - FooterCoordinates.YBottom,
                 FooterCoordinates.LineWidth);
      end;
    {fi}
    AutoPaging := Temp;
    end;
  {fi}
end;  { TPrintPreview.WriteFooter }


procedure TPrintPreview.WriteHeader;
   { If any headers are defined, write them }
var
  I: Integer;
  Temp: Boolean;
  aFlag: Boolean;
begin
  if Dryrun then begin  { Save header with each page }
    SaveCurrentFont;
     { Set 'AutoPaging' off.  Otherwise the header may not get written
       correctly. }
    Temp := AutoPaging;
    AutoPaging := False;
    for I := 1 to HeaderLines do begin
      if (Length(Header[I].Text) > 0) then begin
        with Header[I] do begin
          SetFontInformation(FontName,FontSize,FontStyle);
          aFlag := (Pos('%u',Text) > 0);
          DoWrite(Alignment, inTopMargin + YPosition, Text, aFlag);
          end;
        {od with}
        end;
      {fi}
      end;
    {od for}
    RestoreCurrentFont;
     { Does the user desire a box around the header? }
    if (HeaderCoordinates.Boxed = True) then begin
      if (HeaderCoordinates.Shading > 0) then
        DrawBoxShaded(HeaderCoordinates.XTop,inTopMargin+HeaderCoordinates.YTop,
                       HeaderCoordinates.XBottom,inTopMargin+HeaderCoordinates.YBottom,
                       HeaderCoordinates.LineWidth,HeaderCoordinates.Shading)
      else
        DrawBox(HeaderCoordinates.XTop,inTopMargin+HeaderCoordinates.YTop,
                 HeaderCoordinates.XBottom,inTopMargin+HeaderCoordinates.YBottom,
                 HeaderCoordinates.LineWidth);
      end;
    {fi}
    AutoPaging := Temp;
    end;
  {fi}
end;  { TPrintPreview.WriteHeader }

procedure TPrintPreview.DoWrite(Alignment: Word; Y:Single; Text:string;
                 hfFlag: Boolean);
   { Write some text.  The parameters represent inches from the left ('X')
     and top ('Y') margins. }
var
  X: Single;
begin
  X := 0;
   { How many pixels are there in the inches represented by 'X'? }
  case Alignment of
    wtLeft: X := inLeftMargin;
    wtCenter: X := (PaperWidthInches + inLeftMargin - inRightMargin) / 2;
    wtRight: X := PaperWidthInches - inRightMargin - PixelsToInchesX(OffsetLeft);
    wtIndent: X := inLeftMargin + inCurrentTab;
  end;
  _Write(Alignment,X,Y,Text,[],hfFlag);
end;  { TPrintPreview.DoWrite }

procedure TPrintPreview.WriteText(X:Single; Y:Single; Text:string;
                   Style: TFontStyles);
   { Writes Text with style specified }
begin
   { How many pixels are there in the inches represented by 'X'? }
  if (X = -1) then
    X := LastXPosition
  else if (X = -2) then
    X := inLeftMargin;
  {fi}
  _Write(wtLeft,X,Y,Text,Style,False);
end;  { TPrintPreview.WriteText }

procedure Register;
{ Register this component under the System Tab }
begin
  RegisterComponents('System', [TPrintPreview]);
end;  { Register }

end.

