unit PrintPv1;
{
  This unit contains the preview form which will allow you to view a preview
  of the printed output. It also contains the declarations that store the print
  commands from the PrintDrv module. These commands are stored in a PageList
  for easy viewing and switching between pages. During preview mode, the
  current PageList of commands is sent to the PrintDrv module and "played back"
  to the preview canvas for viewing.  Each of the classes of print commands
  descends from a base class called TMWPrintCommand. (I call my company TMW
  Software, so all the classes start with TMW.  Woody)
}

interface

uses WinTypes, WinProcs, Classes, Graphics, Forms, Controls, Buttons,
  StdCtrls, ExtCtrls, Spin, Mask, SysUtils, Dialogs, DB, DBTables,
  Printers;

const
  iBitmap = 1;

type

  {The CommandSet contains the print commands that can be stored in the
   TMWPrintCommand object list.}
   
  CommandSet =  (commBox,commColumn,commDetailArea,commFont,commFontStyle,
                 commFooterDim,commFooterInf,commGraphic,commGraphicFile,
                 commHeaderDim,commHeaderInf,commLine,commMargins,
                 commNewLine,commOrient,commSetY,commTab,commText,
                 commTopOfPage,commWritePage);

{***********************************************************************}
{                                                                       }
{  TMWPrintCommand is the base class for all print command objects. It  }
{  simply contains the type of command that the object contains so that }
{ the appropriate cast can be performed.                                }
{                                                                       }
{***********************************************************************}

  TMWPrintCommand = class
  private
    fCommand: CommandSet;
  public
    constructor Create(commCommand: CommandSet);
    destructor Destroy; override;
    property Command: CommandSet read fCommand write fCommand;
  end;

{***********************************************************************}
{                                                                       }
{  The TMWPageList object contains the list of page objects created     }
{  during printing.                                                     }
{                                                                       }
{***********************************************************************}

  TMWPageList = class
  private
    Items: TList;
    function GetCount: Integer;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Add(PrintCommand: TMWPrintCommand);
    function Get(Number: Integer): TMWPrintCommand;
    property Count: Integer read GetCount;
  end;

  {***********************************************************************}
  {                                                                       }
  {  TPreviewForm defines the preview form                                }
  {                                                                       }
  {***********************************************************************}

  TPreviewForm = class(TForm)
    Panel2: TPanel;
    BitBtn1: TBitBtn;
    ScrollBox1: TScrollBox;
    ZoomFactor: TComboBox;
    Label2: TLabel;
    PrintBtn: TBitBtn;
    PrintDialog1: TPrintDialog;
    btnFirst: TBitBtn;
    btnPrev: TBitBtn;
    Label1: TLabel;
    edPgNmbr: TMaskEdit;
    StaticText1: TStaticText;
    btnNext: TBitBtn;
    btnLast: TBitBtn;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure ZoomFactorChange(Sender: TObject);
    procedure PrintBtnClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure btnFirstClick(Sender: TObject);
    procedure btnPrevClick(Sender: TObject);
    procedure btnNextClick(Sender: TObject);
    procedure btnLastClick(Sender: TObject);
    procedure edPgNmbrExit(Sender: TObject);
  private
    { Private declarations }
    CurrentPage: Integer; {Used to know which page the preview is viewing}
    NormalPixelsPerInch: Word;
    CanvasOrient: TPrinterOrientation;
    procedure ClearDisplayCanvas;
    procedure EnblBtns;
  public
    { Public declarations }
    StartPercent: Integer;
    { The following provide links to TPrintObject private methods }
    PrtFromPreview: procedure of object;
    PrtPage: procedure(P:TMWPageList; I:Integer) of object;
    SetPrnPtr: procedure(C: TImage) of object;
    GetScl: function: Single of object;
    SetScl: procedure(SF: Single) of object;
    function GetCurrentPage: TImage;
    function GetPageNumber: Integer;
    procedure CopyCurrentCanvas;
    procedure Abort;
    procedure SetOrient(Orient: TPrinterOrientation);
    function GetHeight: Integer;
    function GetWidth: Integer;
  end;  { TPreviewForm }

  {***********************************************************************}
  {                                                                       }
  {  The following declarations are the print command classes descended   }
  {  from TMWPrintCommand. Each one stores the necessary information to   }
  {  execute the printing when played back. This allows the component to  }
  {  make any adjustments needed at run-time depending on the printing    }
  {  device it is using.                                                  }
  {                                                                       }
  {***********************************************************************}

  TMWBox = class(TMWPrintCommand)
  private
    xTop,yTop,xBottom,yBottom: Single;
    lWidth: Single;
    shade: Word;
  public
    constructor Create(x,y,x1,y1: Single;LineWidth: Single;Shading: Word); virtual;
    destructor Destroy; override;
    property X1: Single read xTop;
    property Y1: Single read yTop;
    property X2: Single read xBottom;
    property Y2: Single read yBottom;
    property LineWidth: Single read lWidth;
    property Shading: Word read shade;
  end;

  TMWColumn = class(TMWPrintCommand)
  private
    Numb: Word;
    sPosition,sLength: Single;
  public
    constructor Create(Number: Word;XPosition,Length: Single);virtual;
    destructor Destroy; override;
    property Number: Word read Numb;
    property XPosition: Single read sPosition;
    property Length: Single read sLength;
  end;

  TMWDetail = class(TMWPrintCommand)
  private
    xTop,xBottom: Single;
  public
    constructor Create(x,y: Single);virtual;
    destructor Destroy; override;
    property X: single read xTop;
    property Y: Single read xBottom;
  end;

  TMWFont = class(TMWPrintCommand)
  private
    fName: string;
    fSize: Integer;
    fStyle: TFontStyles;
  public
    constructor Create(Name: string; Size: Integer; Style: TFontStyles);virtual;
    destructor Destroy; override;
    property Name: string read fName;
    property Size: Integer read fSize;
    property Style: TFontStyles read fStyle;
  end;

  TMWFontStyle = class(TMWPrintCommand)
  private
    fStyle: TFontStyles;
  public
    constructor Create(Style: TFontStyles);virtual;
    destructor Destroy; override;
    property Style: TFontStyles read fStyle;
  end;

  TMWFooterDim = class(TMWPrintCommand)
  private
    xTop,yTop,xBottom,yBottom: Single;
    isBox: Boolean;
    lWidth: Single;
    Shade: Word;
  public
    constructor Create(x,y,x1,y1: Single;Boxed: Boolean;LineWidth: Single;
                       Shading: Word);virtual;
    destructor Destroy; override;
    property X1: Single read xTop;
    property Y1: Single read yTop;
    property X2: Single read xBottom;
    property Y2: Single read yBottom;
    property Boxed: Boolean read isBox;
    property LineWidth: Single read lWidth;
    property Shading: Word read shade;
  end;

  TMWFooterInf = class(TMWPrintCommand)
  private
    iLine: Integer;
    yBottom: Single;
    sText,
    sName: string;
    wAlign,
    wSize: Word;
    fStyle: TFontStyles;
  public
    constructor Create(Line: Integer;Alignment: Word;y: Single;Text: string;
                       FontName: string;Size: Word;Style: TFontStyles);virtual;
    destructor Destroy; override;
    property Line: Integer read iLine;
    property yPosition: Single read yBottom;
    property Text: string read sText;
    property Alignment: Word read wAlign;
    property FontName: string read sName;
    property Size: Word read wSize;
    property Style: TFontStyles read fStyle;
  end;

  TMWGraphic = class(TMWPrintCommand)
  private
    xTop,yTop,xHeight,yWidth: Single;
    image: TImage;
    iType: integer;
  public
    constructor Create(x,y,x1,y1: Single;Bitmap: TImage;OfType: Integer);virtual;
    destructor Destroy; override;
    property X: Single read xTop;
    property Y: Single read yTop;
    property Height: Single read xHeight;
    property Width: Single read yWidth;
    property BitMap: TImage read image;
    property ImageType: Integer read iType;
  end;

  TMWHeaderDim = class(TMWPrintCommand)
  private
    xTop,yTop,xBottom,yBottom: Single;
    isBox: Boolean;
    lWidth: Single;
    Shade: Word;
  public
    constructor Create(x,y,x1,y1: Single;Boxed: Boolean;LineWidth: Single;
                       Shading: Word);virtual;
    destructor Destroy; override;
    property X1: Single read xTop;
    property Y1: Single read yTop;
    property X2: Single read xBottom;
    property Y2: Single read yBottom;
    property Boxed: Boolean read isBox;
    property LineWidth: Single read lWidth;
    property Shading: Word read shade;
  end;

  TMWHeaderInf = class(TMWPrintCommand)
  private
    iLine: integer;
    yBottom: Single;
    sText,
    sName: string;
    wAlign,
    wSize: Word;
    fStyle: TFontStyles;
  public
    constructor Create(Line: Integer;Alignment: Word;y: Single;Text: string;
                       FontName: string;Size: Word;Style: TFontStyles);virtual;
    destructor Destroy; override;
    property Line: Integer read iLine;
    property yPosition: Single read yBottom;
    property Text: string read sText;
    property Alignment: Word read wAlign;
    property FontName: string read sName;
    property Size: Word read wSize;
    property Style: TFontStyles read fStyle;
  end;

  TMWLine = class(TMWPrintCommand)
  private
    xTop,yTop,xBottom,yBottom: Single;
    lWidth: Single;
  public
    constructor Create(x,y,x1,y1: Single;LineWidth: Single); virtual;
    destructor Destroy; override;
    property X1: Single read xTop;
    property Y1: Single read yTop;
    property X2: Single read xBottom;
    property Y2: Single read yBottom;
    property LineWidth: Single read lWidth;
  end;

  TMWMargins = class(TMWPrintCommand)
  private
    xTop,yTop,xBottom,yBottom: Single;
  public
    constructor Create(x,y,x1,y1: Single);
    destructor Destroy; override;
    property X1: Single read xTop;
    property Y1: Single read yTop;
    property X2: Single read xBottom;
    property Y2: Single read yBottom;
  end;

  TMWOrient = class(TMWPrintCommand)
  private
    pOrient: TPrinterOrientation;
  public
    constructor Create(Orientation: TPrinterOrientation);virtual;
    destructor Destroy; override;
    property Orientation: TPrinterOrientation read pOrient;
  end;

  TMWSetY = class(TMWPrintCommand)
  private
    yBottom: Single;
  public
    constructor Create(y: Single);virtual;
    destructor Destroy; override;
    property yPosition: Single read yBottom;
  end;

  TMWTab = class(TMWPrintCommand)
  private
    sTab: Single;
  public
    constructor Create(Tab: Single);virtual;
    destructor Destroy; override;
    property Tab: Single read sTab;
  end;

  TMWText = class(TMWPrintCommand)
  private
    wAlign: Word;
    xTop,yTop: Single;
    sText: string;
    bColor: TColor;
    fStyle: TFontStyles;
    aFlag: Boolean;
  public
    constructor Create(Alignment: Word; X,Y: Single; Text: string;
                       Style:TFontStyles; BrushColor: TColor;
                       hfFlag: Boolean);virtual;
    destructor Destroy; override;
    property Alignment: Word read wAlign;
    property X: Single read xTop;
    property Y: Single read yTop;
    property Text: string read sText;
    property Style: TFontStyles read fStyle;
    property BrushColor: TColor read bColor;
    property hfFlag: Boolean read aFlag;
  end;

  {***********************************************************************}
  {                                                                       }
  {  The TMWPages object contains the list of commands for a page created }
  {  during printing. These commands are then played back either by the   }
  {  preview form or by printing to the printer.                          }
  {                                                                       }
  {***********************************************************************}

  TMWPages = class
  private
    PageList: TList;
    CurrentPage: Integer;
    FirstPage,
    LastPage: Integer;
    NewPrinter: Boolean;
    function GetCount: Integer;
  public
    constructor Create;
    destructor Destroy; override;
    procedure NewPage;
    procedure Add(PrintCommand: TMWPrintCommand);
    function Get(Number: Integer): TMWPageList;
    procedure StoreBox(x,y,x1,y1: Single;Width: Single;Shading: Word);
    procedure StoreColumn(Number: Word; Position,Length: Single);
    procedure StoreLine(x,y,x1,y1: Single;Width: Single);
    procedure StoreGeneral(Command: CommandSet);
    procedure StoreGraphic(x,y,h,w: Single;Bitmap: TImage);
    procedure StoreDetailArea(x,x1: Single);
    procedure StoreFontStyle(Style: TFontStyles);
    procedure StoreFontChange(Name: string;Size: Word;Style: TFontStyles);
    procedure StoreFooterDim(x,y,x1,y1: Single;Boxed: Boolean;Width: Single;Shading: Word);
    procedure StoreFooterInf(Line: Integer;Alignment: Word;y: Single;Text: string;
                             FontName: string;FontSize: Word;FontStyle: TFontStyles);
    procedure StoreHeaderDim(x,y,x1,y1: Single;Boxed: Boolean;Width:Single;Shading: Word);
    procedure StoreHeaderInf(Line: Integer;Alignment: Word;y: Single;Text: string;
                             FontName: string;FontSize: Word;FontStyle: TFontStyles);
    procedure StoreMargins(t,b,l,r: Single);
    procedure StoreOrient(Orient: TPrinterOrientation);
    procedure StoreTab(Inches: Single);
    procedure StoreSetY(y: Single);
    procedure StoreText(Alignment: Integer; X,Y: Single; Text: string;
                        Style:TFontStyles; BrushColor: TColor; hfFlag: Boolean);
    function SetPrintOptions: Boolean;
    procedure ClearAll;
    function CountCommands: Integer;
    property FromPage: Integer read FirstPage write FirstPage;
    property ToPage: Integer read LastPage write LastPage;
    property Count: Integer read GetCount;
    property CheckPrinter: Boolean read NewPrinter default False;
  end;

var
  PreviewForm: TPreviewForm;
  NewImage: TImage; {The canvas used for preview modes}
  Pages: TMWPages;
  TotalPages: Integer;
  ScaleAmount: array [0..8] of Single;
  tPageHeight, tPageWidth: Single;

implementation

{$R *.DFM}
uses PrintPrev, PrintPv2;

{***********************************************************************}
{                                                                       }
{  Methods for TMWPages.                                                }
{                                                                       }
{***********************************************************************}

constructor TMWPages.Create;
begin
  PageList := TList.Create;
  CurrentPage := 0;
end;

destructor TMWPages.Destroy;
var
  x: Integer;

begin
  for x := 0 to PageList.Count - 1 do
    TMWPageList(PageList[x]).Free;
  {od for}
  PageList.Free;
  inherited Destroy;
end;

procedure TMWPages.Add(PrintCommand: TMWPrintCommand);
begin
  if CurrentPage = 0 then
    exit;
  {fi}
  TMWPageList(PageList[CurrentPage-1]).Add(PrintCommand);
end;

function TMWPages.Get(Number: Integer): TMWPageList;
begin
  if Number > PageList.Count then
    Result := nil
  else
    Result := TMWPageList(PageList[Number-1]);
  {fi}
end;

procedure TMWPages.NewPage;
begin
  inc(CurrentPage);
  PageList.Add(TMWPageList.Create);
end;

procedure TMWPages.StoreBox(x,y,x1,y1: Single;Width: Single;Shading: Word);
begin
  Add(TMWBox.Create(x,y,x1,y1,Width,Shading));
end;

procedure TMWPages.StoreColumn(Number: Word; Position,Length: Single);
begin
  Add(TMWColumn.Create(Number,Position,Length));
end;

procedure TMWPages.StoreLine(x,y,x1,y1: Single;Width: Single);
begin
  Add(TMWLine.Create(x,y,x1,y1,Width));
end;

procedure TMWPages.StoreGeneral(Command: CommandSet);
begin
  Add(TMWPrintCommand.Create(Command));
end;

procedure TMWPages.StoreGraphic(x,y,h,w: Single;Bitmap: TImage);
begin
  Add(TMWGraphic.Create(x,y,h,w,BitMap,iBitmap));
end;

procedure TMWPages.StoreDetailArea(x,x1: Single);
begin
  Add(TMWDetail.Create(x,x1));
end;

procedure TMWPages.StoreFontStyle(Style: TFontStyles);
begin
  Add(TMWFontStyle.Create(Style));
end;

procedure TMWPages.StoreFontChange(Name: string;Size: Word;Style: TFontStyles);
begin
  Add(TMWFont.Create(Name,Size,Style));
end;

procedure TMWPages.StoreFooterDim(x,y,x1,y1: Single;Boxed: Boolean;
                                  Width: Single; Shading: Word);
begin
  Add(TMWFooterDim.Create(x,y,x1,y1,Boxed,Width,Shading));
end;

procedure TMWPages.StoreFooterInf(Line: Integer;Alignment: Word;y: Single;Text: string;
                                  FontName: string;FontSize: Word;FontStyle: TFontStyles);
begin
  Add(TMWFooterInf.Create(Line,Alignment,y,Text,FontName,FontSize,FontStyle));
end;

procedure TMWPages.StoreHeaderDim(x,y,x1,y1: Single;Boxed: Boolean;
                                  Width: Single; Shading: Word);
begin
  Add(TMWHeaderDim.Create(x,y,x1,y1,Boxed,Width,Shading));
end;

procedure TMWPages.StoreHeaderInf(Line: Integer;Alignment: Word;y: Single;Text: string;
                                  FontName: string;FontSize: Word;FontStyle: TFontStyles);
begin
  Add(TMWHeaderInf.Create(Line,Alignment,y,Text,FontName,FontSize,FontStyle));
end;

procedure TMWPages.StoreMargins(t,b,l,r: Single);
begin
  Add(TMWMargins.Create(t,b,l,r));
end;

procedure TMWPages.StoreOrient(Orient: TPrinterOrientation);
begin
  Add(TMWOrient.Create(Orient));
end;

procedure TMWPages.StoreSetY(y: Single);
begin
  Add(TMWSetY.Create(y));
end;

procedure TMWPages.StoreTab(Inches: Single);
begin
  Add(TMWTab.Create(Inches));
end;

procedure TMWPages.StoreText(Alignment: Integer; X,Y: Single; Text: string;
                             Style:TFontStyles; BrushColor: TColor;
                             hfFlag: Boolean);
begin
  Add(TMWText.Create(Alignment,X,Y,Text,Style,BrushColor,hfFlag));
end;

function TMWPages.GetCount: Integer;
begin
  result := PageList.Count;
end;

procedure TMWPages.ClearAll;
begin
  while PageList.Count > 0 do begin
    TMWPageList(PageList[PageList.Count - 1]).Destroy;
    PageList.Delete(PageList.Count - 1);
    end;
  {od with}
  CurrentPage := 0;
  FirstPage := 0;
  LastPage := 0;
end;

function TMWPages.CountCommands: Integer;
var
  x,
  TotalCommands: Integer;

begin
  TotalCommands := 0;
  for x := FromPage to ToPage do
    inc(TotalCommands,TMWPageList(PageList[x]).Count);
  {od for}
  Result := TotalCommands;
end;

{***********************************************************************}
{                                                                       }
{  Methods for TMWPageList                                              }
{                                                                       }
{***********************************************************************}

constructor TMWPageList.Create;
begin
  Items := TList.Create;
end;

destructor TMWPageList.Destroy;
var
  x: Integer;
begin
  for x := 0 to Items.Count-1 do
    TMWPrintCommand(Items[x]).Free;
  {od for}
  Items.Free;
  inherited Destroy;
end;

procedure TMWPageList.Add(PrintCommand: TMWPrintCommand);
begin
  Items.Add(PrintCommand);
end;

function TMWPageList.Get(Number: Integer): TMWPrintCommand;
begin
  if Number > Items.Count then
    result := nil
  else
    result := TMWPrintCommand(Items[Number-1]);
  {fi}
end;

function TMWPageList.GetCount: Integer;
begin
  result := Items.Count;
end;

{***********************************************************************}
{                                                                       }
{  TMWPrintCommand methods                                              }
{                                                                       }
{***********************************************************************}

constructor TMWPrintCommand.Create(commCommand: CommandSet);
begin
  Command := commCommand;
end;

destructor TMWPrintCommand.Destroy;
begin
  inherited Destroy;
end;

{***********************************************************************}
{                                                                       }
{  The methods for the various command classes follow.                  }
{                                                                       }
{***********************************************************************}

constructor TMWBox.Create(x,y,x1,y1: Single;LineWidth: Single;Shading: Word);
begin
  inherited Create(commBox);
  xTop := x;
  yTop := y;
  xBottom := x1;
  yBottom := y1;
  lWidth := LineWidth;
  shade := Shading;
end;

destructor TMWBox.Destroy;
begin
  inherited Destroy;
end;

constructor TMWLine.Create(x,y,x1,y1: Single;LineWidth: Single);
begin
  inherited Create(commLine);
  xTop := x;
  yTop := y;
  xBottom := x1;
  yBottom := y1;
  lWidth := LineWidth;
end;

destructor TMWLine.Destroy;
begin
  inherited Destroy;
end;

constructor TMWGraphic.Create(x,y,x1,y1: Single;Bitmap: TImage;OfType: Integer);
begin
  inherited Create(commGraphic);
  xTop := x;
  yTop := y;
  xHeight := x1;
  yWidth := y1;
  image := TImage.Create(PreviewForm);
  if image = nil then
    raise Exception.Create('Not enough memory for graphic.');
  {fi}
  image.Height := BitMap.Height;
  image.Width := BitMap.Width;
  image.Picture.Assign(BitMap.Picture);
  iType := OfType;
end;

destructor TMWGraphic.Destroy;
begin
  image.free;
  inherited Destroy;
end;

constructor TMWDetail.Create(x,y: Single);
begin
  inherited Create(commDetailArea);
  xTop := x;
  xBottom := y;
end;

destructor TMWDetail.Destroy;
begin
  inherited Destroy;
end;

constructor TMWFont.Create(Name: string; Size: Integer; Style: TFontStyles);
begin
  inherited Create(commFont);
  fName := Name;
  fSize := Size;
  fStyle := Style;
end;

destructor TMWFont.Destroy;
begin
  inherited Destroy;
end;

constructor TMWFontStyle.Create(Style: TFontStyles);
begin
  inherited Create(commFontStyle);
  fStyle := Style;
end;

destructor TMWFontStyle.Destroy;
begin
  inherited Destroy;
end;

constructor TMWFooterDim.Create(x,y,x1,y1: Single;Boxed: Boolean;LineWidth:Single;
                                Shading: Word);
begin
  inherited Create(commFooterDim);
  xTop := x;
  yTop := y;
  xBottom := x1;
  yBottom := y1;
  isBox := Boxed;
  lWidth := LineWidth;
  shade := Shading;
end;

destructor TMWFooterDim.Destroy;
begin
  inherited Destroy;
end;

constructor TMWFooterInf.Create(Line: Integer;Alignment: Word;y: Single;Text: string;
                                FontName: string;Size: Word;Style: TFontStyles);
begin
  inherited Create(commFooterInf);
  iLine := Line;
  yBottom := y;
  sText := Text;
  wAlign := Alignment;
  sName := FontName;
  wSize := Size;
  fStyle := Style;
end;

destructor TMWFooterInf.Destroy;
begin
  inherited Destroy;
end;

constructor TMWHeaderDim.Create(x,y,x1,y1: Single;Boxed: Boolean;LineWidth: Single;
                                Shading: Word);
begin
  inherited Create(commHeaderDim);
  xTop := x;
  yTop := y;
  xBottom := x1;
  yBottom := y1;
  isBox := Boxed;
  lWidth := LineWidth;
  shade := Shading;
end;

destructor TMWHeaderDim.Destroy;
begin
  inherited Destroy;
end;

constructor TMWHeaderInf.Create(Line: Integer;Alignment: Word;y: Single;Text: string;
                                FontName: string;Size: Word;Style: TFontStyles);
begin
  inherited Create(commHeaderInf);
  iLine := Line;
  yBottom := y;
  sText := Text;
  wAlign := Alignment;
  sName := FontName;
  wSize := Size;
  fStyle := Style;
end;

destructor TMWHeaderInf.Destroy;
begin
  inherited Destroy;
end;

constructor TMWMargins.Create(x,y,x1,y1: Single);
begin
  inherited Create(commMargins);
  xTop := x;
  yTop := y;
  xBottom := x1;
  yBottom := y1;
end;

destructor TMWMargins.Destroy;
begin
  inherited Destroy;
end;

constructor TMWOrient.Create(Orientation: TPrinterOrientation);
begin
  inherited Create(commOrient);
  pOrient := Orientation;
end;

destructor TMWOrient.Destroy;
begin
  inherited Destroy;
end;

constructor TMWSetY.Create(y: Single);
begin
  inherited Create(commSetY);
  yBottom := y;
end;

destructor TMWSetY.Destroy;
begin
  inherited Destroy;
end;

constructor TMWTab.Create(Tab: Single);
begin
  inherited Create(commTab);
  sTab := Tab;
end;

destructor TMWTab.Destroy;
begin
  inherited Destroy;
end;

constructor TMWText.Create(Alignment: Word; X,Y: Single; Text: string;
                  Style:TFontStyles; BrushColor: TColor; hfFlag: Boolean);
begin
  inherited Create(commText);
  wAlign := Alignment;
  xTop := x;
  yTop := y;
  sText := Text;
  fStyle := Style;
  bColor := BrushColor;
  aFlag := hfFlag;
end;

destructor TMWText.Destroy;
begin
  inherited Destroy;
end;

constructor TMWColumn.Create(Number: Word;XPosition,Length: Single);
begin
  inherited Create(commColumn);
  Numb := Number;
  sPosition := XPosition;
  sLength := Length;
end;

destructor TMWColumn.Destroy;
begin
  inherited Destroy;
end;

{***********************************************************************}
{                                                                       }
{  Methods for TPreviewForm                                             }
{                                                                       }
{***********************************************************************}


procedure TPreviewForm.Abort;
begin
   CurrentPage := 0;
   TotalPages := 0;
   ClearDisplayCanvas;
end;  { TPreviewForm.Abort }

procedure TPreviewForm.ClearDisplayCanvas;
// Procedure to clean off the preview canvas such as when changing pages.
var
  cTemp: TColor;
  ScaleFactor: Single;
begin
  ScaleFactor := ScaleAmount[ZoomFactor.ItemIndex];
  cTemp := NewImage.Canvas.Brush.Color;
  NewImage.Canvas.Brush.Color := clWhite;
  NewImage.Canvas.FillRect(rect(0,0,
       Round(GetWidth * ScaleFactor),
       Round(GetHeight * ScaleFactor)));
  NewImage.Canvas.Brush.Color := cTemp;
end;  { TPreviewForm.ClearDisplayCanvas }

procedure TPreviewForm.CopyCurrentCanvas;
// This procedure sets the current printing canvas to the preview canvas and
// calls the procedure to play out the page commands.
begin
  try
    ClearDisplayCanvas;
    SetPrnPtr(NewImage);
    PrtPage(Pages.Get(CurrentPage),CurrentPage);
  except
    ShowMessage('There was an error copying zoomed images.');
    raise;
  end;
end;  { TPreviewForm.CopyCurrentCanvas }

procedure TPreviewForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  try
    NewImage.Free;
  finally
    Action := caFree;
    PreviewForm := nil;
  end;
end;  { TPreviewForm.FormClose }

procedure TPreviewForm.FormCreate(Sender: TObject);
begin
  CurrentPage := 1;
  CanvasOrient := poPortrait;
  NewImage := TImage.Create(Self);
  NewImage.Name := 'DisplayPage';
  NewImage.Parent := ScrollBox1;
  NewImage.Top := 10;
  NewImage.Left := 10; 
  NewImage.Height := 1500;
  NewImage.Width := 1500;
  NormalPixelsPerInch := NewImage.Canvas.Font.PixelsPerInch;
  NewImage.Visible := true;
  ScaleAmount[0] := 0.4;
  ScaleAmount[1] := 0.5;
  ScaleAmount[2] := 0.6;
  ScaleAmount[3] := 0.7;
  ScaleAmount[4] := 0.8;
  ScaleAmount[5] := 0.9;
  ScaleAmount[6] := 1.0;
end;  { TPreviewForm.FormCreate }

procedure TPreviewForm.FormShow(Sender: TObject);
var
  BoxWidth, BoxHeight: Extended;
begin
  BoxWidth := (ScrollBox1.ClientWidth - 2*NewImage.Left -
               ScrollBox1.VertScrollBar.Size)/PixelsPerInch;
  BoxHeight := (ScrollBox1.ClientHeight - 2*NewImage.Top -
                ScrollBox1.HorzScrollBar.Size)/PixelsPerInch;
  case CanvasOrient of
    poPortrait: begin
      ScaleAmount[7] := BoxWidth/tPageWidth;
      ScaleAmount[8] := BoxHeight/tPageHeight;
      end;
    poLandscape: begin
      ScaleAmount[7] := BoxWidth/tPageHeight;
      ScaleAmount[8] := BoxHeight/tPageWidth;
      end;
  end{case};
  if ScaleAmount[8] > ScaleAmount[7] then
    ScaleAmount[8] := ScaleAmount[7];
  {fi}
  ZoomFactor.ItemIndex := StartPercent;
  CurrentPage := 1;
  StaticText1.Caption := 'of '+IntToStr(TotalPages)+' Pages';
  ZoomFactorChange(ZoomFactor);
  EnblBtns;
end;  { TPreviewForm.FormShow }

function TPreviewForm.GetCurrentPage: TImage;
// Function to return instance of the preview canvas. Originally, I was working
// with multiple images, one for each page, but this proved to be much more costly.
// That's when I decided to go with storing the print commands. I left this function
// in just in case there is a need for it later. Besides, the PrintDrv module
// already uses it and so it was just easier to leave it.
begin
  try
    result := NewImage;
  except
    ShowMessage('Error finding preview image.');
    raise;
  end;
end;  { TPreviewForm.GetCurrentPage }

function TPreviewForm.GetPageNumber: Integer;
// Returns the current page number.
begin
  Result := CurrentPage;
end;  { TPreviewForm.GetPageNumber }

procedure TPreviewForm.ZoomFactorChange(Sender: TObject);
// This procedure adjusts the size of the preview canvas depending on the
// scale value selected.
var
  ScaleFactor: Single;
begin
  ScaleFactor := ScaleAmount[ZoomFactor.ItemIndex];
  NewImage.Hide;
  case CanvasOrient of
    poPortrait: begin
      NewImage.Width := round(tPageWidth * NormalPixelsPerInch * ScaleFactor);
      NewImage.Height := round(tPageHeight * NormalPixelsPerInch * ScaleFactor);
      end;
    poLandscape: begin
      NewImage.Width := round(tPageHeight * NormalPixelsPerInch * ScaleFactor);
      NewImage.Height := round(tPageWidth * NormalPixelsPerInch * ScaleFactor);
      end;
  end;
  NewImage.Canvas.Font.PixelsPerInch := trunc(NormalPixelsPerInch * ScaleFactor);
  SetScl(Scalefactor);
  CopyCurrentCanvas;
  NewImage.show;
end;  { TPreviewForm.ZoomFactorChange }

procedure TPreviewForm.PrintBtnClick(Sender: TObject);
// Print from the preview form when user click Print button.
var
	CurrentScale: Single;
begin
  try
    Screen.Cursor := crHourGlass;
    CurrentScale := GetScl;
    SetScl(1.0);
    PrtFromPreview;
    SetScl(CurrentScale);
  finally
    Screen.Cursor := crDefault;
  end;
end;  { TPreviewForm.PrintBtnClick }

procedure TPreviewForm.SetOrient(Orient: TPrinterOrientation);
// Set the orientation of the preview canvas.
begin
  CanvasOrient := Orient;
end;  { TPreviewForm.SetOrient }

// Since the preview canvas must be set to maximum height and width when created,
// the standard Width and Height properties always return the maximum amount so
// they had to be circumvented with the following two functions:
// GetHeight and GetWidth

function TPreviewForm.GetHeight: Integer;
begin
  if CanvasOrient = poLandscape then
    Result := Trunc(tPageWidth * NormalPixelsPerInch)
  else
    Result := Trunc(tPageHeight * NormalPixelsPerInch);
  {fi}
end;  { TPreviewForm.GetHeight }

function TPreviewForm.GetWidth: Integer;
begin
  if CanvasOrient = poLandscape then
    Result := Trunc(tPageHeight * NormalPixelsPerInch)
  else
    Result := Trunc(tPageWidth * NormalPixelsPerInch);
  {fi}
end;  { TPreviewForm.GetWidth }

function TMWPages.SetPrintOptions: Boolean;
// This function displays the print dialog box after setting the pages available.
// If the users cancels the dialog box, the function returns false, otherwise it
// sets the values for printing a range of pages which default to all pages.
var
  SavePrinterIndex: Integer;
begin
  Result := true;
  SavePrinterIndex := Printer.PrinterIndex;
  with PreviewForm.PrintDialog1 do begin
    Options := [poPageNums];
    MinPage := 1;
    FromPage := 1;
    ToPage := TotalPages;
    MaxPage := TotalPages;
    if Not Execute then
      Result := false
    else begin
      FirstPage := FromPage;
      LastPage := ToPage;
      NewPrinter := SavePrinterIndex <> Printer.PrinterIndex;
      end;
    {fi}
    end;
  {od with}
end;  { TMWPages.SetPrintOptions }

procedure TPreviewForm.FormResize(Sender: TObject);
var
  x: Integer;
  BoxWidth, BoxHeight: Extended;
begin
  BoxWidth := (ScrollBox1.ClientWidth - 2*NewImage.Left -
               ScrollBox1.VertScrollBar.Size)/PixelsPerInch;
  BoxHeight := (ScrollBox1.ClientHeight - 2*NewImage.Top -
                ScrollBox1.HorzScrollBar.Size)/PixelsPerInch;
  case CanvasOrient of
    poPortrait: begin
      ScaleAmount[7] := BoxWidth/tPageWidth;
      ScaleAmount[8] := BoxHeight/tPageHeight;
      end;
    poLandscape: begin
      ScaleAmount[7] := BoxWidth/tPageHeight;
      ScaleAmount[8] := BoxHeight/tPageWidth;
      end;
  end{case};
  if ScaleAmount[8] > ScaleAmount[7] then
    ScaleAmount[8] := ScaleAmount[7];
  {fi}
  x := 40;
  if NewImage.Height + 30 > ScrollBox1.Height then
    ScrollBox1.VertScrollBar.Range := NewImage.Height + x;
  {fi}
  if NewImage.Width + 30 > ScrollBox1.Width then
    ScrollBox1.HorzScrollBar.Range := NewImage.Width + x;
  {fi}
  ZoomFactorChange(Sender);
end;  { TPreviewForm.FormResize }

procedure TPreviewForm.EnblBtns;
var
  S: String;
  PageNmbr: Integer;
begin
  S := Trim(edPgNmbr.Text);
  PageNmbr := StrToInt(S);
  btnFirst.Enabled := PageNmbr > 1;;
  btnPrev.Enabled := btnFirst.Enabled;
  btnLast.Enabled := PageNmbr < TotalPages;
  btnNext.Enabled := btnLast.Enabled;
end;  { TPreviewForm.EnblBtns }

procedure TPreviewForm.btnFirstClick(Sender: TObject);
begin
  edPgNmbr.Text := '  1';
  CurrentPage := 1;
  ScrollBox1.VertScrollBar.Position := 0;
  CopyCurrentCanvas;
  EnblBtns;
end;  { TPreviewForm.btnFirstClick }

procedure TPreviewForm.btnPrevClick(Sender: TObject);
var
  PageNmbr: Integer;
  S: string;
begin
  S := Trim(edPgNmbr.Text);
  PageNmbr := StrToInt(S);
  Dec(PageNmbr);
  CurrentPage := PageNmbr;
  ScrollBox1.VertScrollBar.Position := 0;
  CopyCurrentCanvas;
  Str(PageNmbr:3,S);
  edPgNmbr.Text := S;
  EnblBtns;
end;  { TPreviewForm.btnPrevClick }

procedure TPreviewForm.btnNextClick(Sender: TObject);
var
  PageNmbr: Integer;
  S: string;
begin
  S := Trim(edPgNmbr.Text);
  PageNmbr := StrToInt(S);
  Inc(PageNmbr);
  CurrentPage := PageNmbr;
  ScrollBox1.VertScrollBar.Position := 0;
  CopyCurrentCanvas;
  Str(PageNmbr:3,S);
  edPgNmbr.Text := S;
  EnblBtns;
end;  { TPreviewForm.btnNextClick }

procedure TPreviewForm.btnLastClick(Sender: TObject);
var
  S: string;
begin
  Str(TotalPages:3,S);
  edPgNmbr.Text := S;
  CurrentPage := TotalPages;
  ScrollBox1.VertScrollBar.Position := 0;
  CopyCurrentCanvas;
  EnblBtns;
end;  { TPreviewForm.btnLastClick }

procedure TPreviewForm.edPgNmbrExit(Sender: TObject);
var
  PageNmbr: Integer;
  S: string;
begin
  S := Trim(edPgNmbr.Text);
  if Length(S) > 0 then
    try
      PageNmbr := StrToInt(S);
    except
      PageNmbr := 1;
    end
  else
    PageNmbr := 1;
  {fi}
  if PageNmbr > TotalPages then
    PageNmbr := TotalPages
  else if PageNmbr < 1 then
    PageNmbr := 1;
  {fi}
  Str(PageNmbr:3,S);
  edPgNmbr.Text := S;
  CurrentPage := PageNmbr;
  CopyCurrentCanvas;
  EnblBtns;
end;  { TPreviewForm.edPgNmbrExit }

end.
